############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Hugo <hugovk@users.noreply.github.com>                        #
# Copyright 2018 R1kk3r <R1kk3r@users.noreply.github.com>                      #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Jonathan Kliem <jonathan.kliem@gmail.com>                     #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import github
from github.Auth import Login

from . import Framework
from .Authentication import CustomAuth


class Logging(Framework.BasicTestCase):
    class MockLogger:
        def __init__(self):
            self.verb = None
            self.url = None
            self.requestHeaders = None
            self.input = None
            self.status = None
            self.responseHeaders = None
            self.output = None

        def isEnabledFor(self, kind):
            return True

        def debug(
            self,
            format_string,
            verb,
            scheme,
            hostname,
            fragment,
            requestHeaders,
            input_,
            status,
            responseHeaders,
            output,
        ):
            self.verb = verb
            self.url = f"{scheme}://{hostname}{fragment}"
            self.requestHeaders = requestHeaders
            self.input = input_
            self.status = status
            self.responseHeaders = responseHeaders
            self.output = output

    def setUp(self):
        super().setUp()
        self.logger = self.MockLogger()
        github.Requester.Requester.injectLogger(self.logger)

    def tearDown(self):
        github.Requester.Requester.resetLogger()
        super().tearDown()

    def assertLogging(self, verb, url, requestHeaders, responseHeaders, output):
        self.assertEqual(self.logger.verb, verb)
        self.assertEqual(self.logger.url, url)
        self.assertEqual(self.logger.requestHeaders, requestHeaders)
        self.assertIsNone(self.logger.input)
        self.assertEqual(self.logger.status, 200)
        self.assertEqual(self.logger.responseHeaders, responseHeaders)
        self.assertEqual(self.logger.output, output)

    def testLoggingWithBasicAuthentication(self):
        self.assertEqual(github.Github(auth=Login("login", "password")).get_user().name, "Vincent Jacques")
        url = "https://api.github.com/user"
        requestHeaders = {
            "Authorization": "Basic (login and password removed)",
            "User-Agent": "PyGithub/Python",
        }
        responseHeaders = {
            "status": "200 OK",
            "content-length": "806",
            "x-github-media-type": "github.beta; format=json",
            "x-content-type-options": "nosniff",
            "x-ratelimit-limit": "5000",
            "vary": "Accept, Authorization, Cookie",
            "x-ratelimit-remaining": "4993",
            "server": "nginx",
            "last-modified": "Fri, 14 Sep 2012 18:47:46 GMT",
            "connection": "keep-alive",
            "etag": '"434dfe5d3f50558fe3cea087cb95c401"',
            "cache-control": "private, s-maxage=60, max-age=60",
            "date": "Mon, 17 Sep 2012 17:12:32 GMT",
            "content-type": "application/json; charset=utf-8",
            "DEBUG_FRAME": 0,
        }
        output = '{"owned_private_repos":3,"disk_usage":18612,"following":28,"type":"User","public_repos":13,"location":"Paris, France","company":"Criteo","avatar_url":"https://secure.gravatar.com/avatar/b68de5ae38616c296fa345d2b9df2225?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-user-420.png","plan":{"space":614400,"private_repos":5,"name":"micro","collaborators":1},"blog":"http://vincent-jacques.net","login":"jacquev6","public_gists":3,"html_url":"https://github.com/jacquev6","hireable":false,"created_at":"2010-07-09T06:10:06Z","private_gists":5,"followers":13,"name":"Vincent Jacques","email":"vincent@vincent-jacques.net","bio":"","total_private_repos":3,"collaborators":0,"gravatar_id":"b68de5ae38616c296fa345d2b9df2225","id":327146,"url":"https://api.github.com/users/jacquev6"}'
        self.assertLogging("GET", url, requestHeaders, responseHeaders, output)

    def testLoggingWithOAuthAuthentication(self):
        self.assertEqual(github.Github(auth=self.oauth_token).get_user().name, "Vincent Jacques")
        url = "https://api.github.com/user"
        requestHeaders = {
            "Authorization": "token (oauth token removed)",
            "User-Agent": "PyGithub/Python",
        }
        responseHeaders = {
            "status": "200 OK",
            "x-ratelimit-remaining": "4993",
            "x-github-media-type": "github.beta; format=json",
            "x-content-type-options": "nosniff",
            "vary": "Accept, Authorization, Cookie",
            "content-length": "628",
            "server": "nginx",
            "last-modified": "Tue, 25 Sep 2012 07:42:42 GMT",
            "connection": "keep-alive",
            "x-ratelimit-limit": "5000",
            "etag": '"c23ad6b5815fc3d6ec6341c4a47afe85"',
            "cache-control": "private, max-age=60, s-maxage=60",
            "date": "Tue, 25 Sep 2012 20:36:54 GMT",
            "x-oauth-scopes": "",
            "content-type": "application/json; charset=utf-8",
            "x-accepted-oauth-scopes": "user",
            "DEBUG_FRAME": 0,
        }
        output = '{"type":"User","bio":"","html_url":"https://github.com/jacquev6","login":"jacquev6","followers":14,"company":"Criteo","blog":"http://vincent-jacques.net","public_repos":13,"created_at":"2010-07-09T06:10:06Z","avatar_url":"https://secure.gravatar.com/avatar/b68de5ae38616c296fa345d2b9df2225?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-user-420.png","email":"vincent@vincent-jacques.net","following":29,"name":"Vincent Jacques","gravatar_id":"b68de5ae38616c296fa345d2b9df2225","hireable":false,"id":327146,"public_gists":3,"location":"Paris, France","url":"https://api.github.com/users/jacquev6"}'
        self.assertLogging("GET", url, requestHeaders, responseHeaders, output)

    def testLoggingWithoutAuthentication(self):
        self.assertEqual(github.Github().get_user("jacquev6").name, "Vincent Jacques")
        url = "https://api.github.com/users/jacquev6"
        requestHeaders = {"User-Agent": "PyGithub/Python"}
        responseHeaders = {
            "status": "200 OK",
            "content-length": "628",
            "x-github-media-type": "github.beta; format=json",
            "x-content-type-options": "nosniff",
            "x-ratelimit-limit": "5000",
            "vary": "Accept",
            "x-ratelimit-remaining": "4989",
            "server": "nginx",
            "last-modified": "Tue, 25 Sep 2012 07:42:42 GMT",
            "connection": "keep-alive",
            "etag": '"9bd085221a16b6d2ea95e72634c3c1ac"',
            "cache-control": "public, max-age=60, s-maxage=60",
            "date": "Tue, 25 Sep 2012 20:38:56 GMT",
            "content-type": "application/json; charset=utf-8",
            "DEBUG_FRAME": 0,
        }
        output = '{"type":"User","html_url":"https://github.com/jacquev6","login":"jacquev6","followers":14,"company":"Criteo","created_at":"2010-07-09T06:10:06Z","email":"vincent@vincent-jacques.net","hireable":false,"avatar_url":"https://secure.gravatar.com/avatar/b68de5ae38616c296fa345d2b9df2225?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-user-420.png","public_gists":3,"bio":"","following":29,"name":"Vincent Jacques","blog":"http://vincent-jacques.net","gravatar_id":"b68de5ae38616c296fa345d2b9df2225","id":327146,"public_repos":13,"location":"Paris, France","url":"https://api.github.com/users/jacquev6"}'
        self.assertLogging("GET", url, requestHeaders, responseHeaders, output)

    def testLoggingWithBaseUrl(self):
        # ReplayData forged, not recorded
        self.assertEqual(
            github.Github(base_url="http://my.enterprise.com/my/prefix").get_user("jacquev6").name,
            "Vincent Jacques",
        )
        url = "http://my.enterprise.com/my/prefix/users/jacquev6"
        requestHeaders = {"User-Agent": "PyGithub/Python"}
        responseHeaders = {
            "status": "200 OK",
            "content-length": "628",
            "x-github-media-type": "github.beta; format=json",
            "x-content-type-options": "nosniff",
            "x-ratelimit-limit": "5000",
            "vary": "Accept",
            "x-ratelimit-remaining": "4989",
            "server": "nginx",
            "last-modified": "Tue, 25 Sep 2012 07:42:42 GMT",
            "connection": "keep-alive",
            "etag": '"9bd085221a16b6d2ea95e72634c3c1ac"',
            "cache-control": "public, max-age=60, s-maxage=60",
            "date": "Tue, 25 Sep 2012 20:38:56 GMT",
            "content-type": "application/json; charset=utf-8",
            "DEBUG_FRAME": 0,
        }
        output = '{"type":"User","html_url":"https://github.com/jacquev6","login":"jacquev6","followers":14,"company":"Criteo","created_at":"2010-07-09T06:10:06Z","email":"vincent@vincent-jacques.net","hireable":false,"avatar_url":"https://secure.gravatar.com/avatar/b68de5ae38616c296fa345d2b9df2225?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-user-420.png","public_gists":3,"bio":"","following":29,"name":"Vincent Jacques","blog":"http://vincent-jacques.net","gravatar_id":"b68de5ae38616c296fa345d2b9df2225","id":327146,"public_repos":13,"location":"Paris, France","url":"https://api.github.com/users/jacquev6"}'
        self.assertLogging("GET", url, requestHeaders, responseHeaders, output)

    def testLoggingDoesNotModifyRequestHeaders(self):
        # Recorded replay data already sanitizes Authorization headers, so we
        # need to go under the covers
        requestHeaders = {"Authorization": "thisisnotatoken"}
        responseHeaders = {"status": "200 OK"}
        github.Github()._Github__requester._Requester__log(
            "GET",
            "http://example.com",
            requestHeaders,
            None,
            200,
            responseHeaders,
            None,
        )
        self.assertEqual(requestHeaders["Authorization"], "thisisnotatoken")

    def testMaskingOfCustomAuthHeader(self):
        requestHeaders = {"Custom key": "secret"}
        responseHeaders = {"status": "200 OK"}
        github.Github(auth=CustomAuth())._Github__requester._Requester__log(
            "GET",
            "http://example.com",
            requestHeaders,
            None,
            200,
            responseHeaders,
            None,
        )
        self.assertEqual({"Custom key": "Masked custom header"}, self.logger.requestHeaders)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 karsten-wagner <39054096+karsten-wagner@users.noreply.github.com>#
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Caleb McCombs <caleb@mccombalot.net>                          #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class SecurityAndAnalysis(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("PyGithub/PyGithub")
        self.maxDiff = None

    def testAttributes(self):
        security_and_analysis = self.repo.security_and_analysis
        self.assertIsNone(security_and_analysis.advanced_security)
        self.assertEqual(security_and_analysis.dependabot_security_updates.status, "enabled")
        self.assertEqual(security_and_analysis.secret_scanning.status, "disabled")
        self.assertIsNone(security_and_analysis.secret_scanning_ai_detection)
        self.assertEqual(security_and_analysis.secret_scanning_push_protection.status, "disabled")
        self.assertEqual(security_and_analysis.secret_scanning_non_provider_patterns.status, "disabled")
        self.assertIsNone(security_and_analysis.secret_scanning_validity_checks)

    def testRepresentation(self):
        self.assertEqual(
            repr(self.repo.security_and_analysis),
            "SecurityAndAnalysis("
            'secret_scanning_validity_checks="None", '
            'secret_scanning_push_protection="SecurityAndAnalysisFeature(status="disabled")", '
            'secret_scanning_non_provider_patterns="SecurityAndAnalysisFeature(status="disabled")", '
            'secret_scanning="SecurityAndAnalysisFeature(status="disabled")", '
            'dependabot_security_updates="SecurityAndAnalysisFeature(status="enabled")", '
            'advanced_security="None")',
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Hemslo Wang <hemslo.wang@gmail.com>                           #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Trim21 <trim21.me@gmail.com>                                  #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Jakub Smolar <jakub.smolar@scylladb.com>                      #
# Copyright 2025 Timothy Klopotoski <tklopotoski@ebsco.com>                    #
# Copyright 2026 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import contextlib
from datetime import datetime, timedelta, timezone
from unittest import mock

import github
from github import Requester as gr

from . import Framework
from .GithubIntegration import APP_ID, PRIVATE_KEY

REPO_NAME = "PyGithub/PyGithub"


class Requester(Framework.TestCase):
    logger = None

    def setUp(self):
        super().setUp()
        self.logger = mock.MagicMock()
        github.Requester.Requester.injectLogger(self.logger)

    def tearDown(self):
        github.Requester.Requester.resetLogger()
        super().tearDown()

    def testRecreation(self):
        class TestAuth(github.Auth.AppAuth):
            pass

        # create a Requester with non-default arguments
        auth = TestAuth(123, "key")
        requester = github.Requester.Requester(
            auth=auth,
            base_url="https://base.url",
            timeout=1,
            user_agent="user agent",
            per_page=123,
            verify=False,
            retry=3,
            pool_size=5,
            seconds_between_requests=1.2,
            seconds_between_writes=3.4,
            # v3: this should not be the default value, so if this has been changed in v3,
            # change it here is well
            lazy=True,
        )
        kwargs = requester.kwargs

        # assert kwargs consists of ALL constructor arguments
        self.assertEqual(kwargs.keys(), github.Requester.Requester.__init__.__annotations__.keys())
        self.assertEqual(
            kwargs,
            dict(
                auth=auth,
                base_url="https://base.url",
                timeout=1,
                user_agent="user agent",
                per_page=123,
                verify=False,
                retry=3,
                pool_size=5,
                seconds_between_requests=1.2,
                seconds_between_writes=3.4,
                lazy=True,
            ),
        )

        # create a copy Requester, assert identity via kwargs
        copy = github.Requester.Requester(**kwargs)
        self.assertEqual(copy.kwargs, kwargs)

        # create Github instance, assert identity requester
        gh = github.Github(**kwargs)
        self.assertEqual(gh._Github__requester.kwargs, kwargs)

        # create GithubIntegration instance, assert identity requester
        gi = github.GithubIntegration(**kwargs)
        self.assertEqual(gi._GithubIntegration__requester.kwargs, kwargs)

    def testWithAuth(self):
        class TestAuth(github.Auth.AppAuth):
            pass

        # create a Requester with non-default arguments
        auth = TestAuth(123, "key")
        requester = github.Requester.Requester(
            auth=auth,
            base_url="https://base.url",
            timeout=1,
            user_agent="user agent",
            per_page=123,
            verify=False,
            retry=3,
            pool_size=5,
            seconds_between_requests=1.2,
            seconds_between_writes=3.4,
            # v3: this should not be the default value, so if this has been changed in v3,
            # change it here is well
            lazy=True,
        )

        # create a copy with different auth
        auth2 = TestAuth(456, "key2")
        copy = requester.withAuth(auth2)

        # assert kwargs of copy
        self.assertEqual(
            copy.kwargs,
            dict(
                auth=auth2,
                base_url="https://base.url",
                timeout=1,
                user_agent="user agent",
                per_page=123,
                verify=False,
                retry=3,
                pool_size=5,
                seconds_between_requests=1.2,
                seconds_between_writes=3.4,
                lazy=True,
            ),
        )

    def testGetParametersOfUrl(self):
        self.assertEqual({}, gr.Requester.get_parameters_of_url("https://github.com/api"))
        self.assertEqual({"per_page": ["10"]}, gr.Requester.get_parameters_of_url("https://github.com/api?per_page=10"))
        self.assertEqual(
            {"per_page": ["10"], "page": ["2"]},
            gr.Requester.get_parameters_of_url("https://github.com/api?per_page=10&page=2"),
        )
        self.assertEqual(
            {"item": ["1", "2", "3"]}, gr.Requester.get_parameters_of_url("https://github.com/api?item=1&item=2&item=3")
        )

    def testAddParametersToUrl(self):
        self.assertEqual("https://github.com/api", gr.Requester.add_parameters_to_url("https://github.com/api", {}))
        self.assertEqual(
            "https://github.com/api?per_page=10",
            gr.Requester.add_parameters_to_url("https://github.com/api", {"per_page": 10}),
        )
        self.assertEqual(
            "https://github.com/api?page=2&per_page=10",
            gr.Requester.add_parameters_to_url("https://github.com/api", {"per_page": 10, "page": 2}),
        )
        self.assertEqual(
            "https://github.com/api?page=2&per_page=10",
            gr.Requester.add_parameters_to_url("https://github.com/api?per_page=10", {"page": 2}),
        )
        self.assertEqual(
            "https://github.com/api?page=2&per_page=10",
            gr.Requester.add_parameters_to_url("https://github.com/api?per_page=10&page=1", {"page": 2}),
        )
        self.assertEqual(
            "https://github.com/api?item=3&item=4",
            gr.Requester.add_parameters_to_url("https://github.com/api?item=1&item=2&item=3", {"item": [3, 4]}),
        )

    def testCloseGithub(self):
        mocked_connection = mock.MagicMock()
        mocked_custom_connection = mock.MagicMock()

        with github.Github() as gh:
            requester = gh._Github__requester
            requester._Requester__connection = mocked_connection
            requester._Requester__custom_connections.append(mocked_custom_connection)

        mocked_connection.close.assert_called_once_with()
        mocked_custom_connection.close.assert_called_once_with()
        self.assertIsNone(requester._Requester__connection)

    def testCloseGithubIntegration(self):
        mocked_connection = mock.MagicMock()
        mocked_custom_connection = mock.MagicMock()

        auth = github.Auth.AppAuth(APP_ID, PRIVATE_KEY)
        with github.GithubIntegration(auth=auth) as gi:
            requester = gi._GithubIntegration__requester
            requester._Requester__connection = mocked_connection
            requester._Requester__custom_connections.append(mocked_custom_connection)

        mocked_connection.close.assert_called_once_with()
        mocked_custom_connection.close.assert_called_once_with()
        self.assertIsNone(requester._Requester__connection)

    def testLoggingRedirection(self):
        self.assertEqual(self.g.get_repo("EnricoMi/test").name, "test-renamed")
        self.logger.info.assert_called_once_with(
            "Following Github server redirection from /repos/EnricoMi/test to /repositories/638123443"
        )

    def testBaseUrlSchemeRedirection(self):
        gh = github.Github(base_url="http://api.github.com")
        with self.assertRaises(RuntimeError) as exc:
            gh.get_repo("PyGithub/PyGithub")
        self.assertEqual(
            exc.exception.args,
            (
                "Github server redirected from http protocol to https, please correct your "
                "Github server URL via base_url: Github(base_url=...)",
            ),
        )

    def testBaseUrlHostRedirection(self):
        gh = github.Github(base_url="https://www.github.com")
        with self.assertRaises(RuntimeError) as exc:
            gh.get_repo("PyGithub/PyGithub")
        self.assertEqual(
            exc.exception.args,
            (
                "Github server redirected from host www.github.com to github.com, "
                "please correct your Github server URL via base_url: Github(base_url=...)",
            ),
        )

    def testBaseUrlPortRedirection(self):
        # replay data forged
        gh = github.Github(base_url="https://api.github.com")
        with self.assertRaises(RuntimeError) as exc:
            gh.get_repo("PyGithub/PyGithub")
        self.assertEqual(
            exc.exception.args,
            (
                "Requested https://api.github.com/repos/PyGithub/PyGithub but server "
                "redirected to https://api.github.com:443/repos/PyGithub/PyGithub, "
                "you may need to correct your Github server URL "
                "via base_url: Github(base_url=...)",
            ),
        )

    def testBaseUrlPrefixRedirection(self):
        # replay data forged
        gh = github.Github(base_url="https://api.github.com/api/v3")
        self.assertEqual(gh.get_repo("PyGithub/PyGithub").name, "PyGithub")
        self.logger.info.assert_called_once_with(
            "Following Github server redirection from /api/v3/repos/PyGithub/PyGithub to /repos/PyGithub/PyGithub"
        )

    def testHostnameHasDomain(self):
        assert self.g.requester.__hostnameHasDomain("github.com", "github.com")
        assert self.g.requester.__hostnameHasDomain("api.github.com", "github.com")
        assert self.g.requester.__hostnameHasDomain("api.github.com", "github.com")
        assert self.g.requester.__hostnameHasDomain("ghe.local", "ghe.local")
        assert self.g.requester.__hostnameHasDomain("api.ghe.local", "ghe.local")
        assert self.g.requester.__hostnameHasDomain("api.prod.ghe.local", "prod.ghe.local")
        assert self.g.requester.__hostnameHasDomain("github.com", ["github.com", "githubusercontent.com"])
        assert self.g.requester.__hostnameHasDomain("api.github.com", ["github.com", "githubusercontent.com"])
        assert self.g.requester.__hostnameHasDomain("githubusercontent.com", ["github.com", "githubusercontent.com"])
        assert self.g.requester.__hostnameHasDomain(
            "objects.githubusercontent.com", ["github.com", "githubusercontent.com"]
        )
        assert self.g.requester.__hostnameHasDomain("maliciousgithub.com", "github.com") is False
        assert self.g.requester.__hostnameHasDomain("abc.def", ["github.com", "githubusercontent.com"]) is False

    def testAssertUrlAllowed(self):
        # default github.com requester
        requester = self.g.requester
        self.assertEqual(set(requester.__domains), {"github.com", "githubusercontent.com"})

        for allowed in [
            "https://api.github.com/request",
            "https://github.com/path",
            "https://uploads.github.com/path",
            "https://status.github.com/path",
            "https://githubusercontent.com/path",
            "https://objects.githubusercontent.com/path",
            "https://release-assets.githubusercontent.com/path",
        ]:
            requester.__assertUrlAllowed(allowed)

        for not_allowed, arg in [
            ("https://prod.ghe.local/github-api/request", "prod.ghe.local"),
            ("https://api.prod.ghe.local/github-api/request", "api.prod.ghe.local"),
            ("https://uploads.prod.ghe.local/github-api/path", "uploads.prod.ghe.local"),
            ("https://status.prod.ghe.local/github-api/path", "status.prod.ghe.local"),
            ("https://example.com/", "example.com"),
        ]:
            with self.assertRaises(AssertionError) as exc:
                requester.__assertUrlAllowed(not_allowed)
            self.assertEqual(exc.exception.args, (arg,))

        # custom (Enterprise) requester with prefix
        requester = github.Github(base_url="https://prod.ghe.local/github-api/").requester
        self.assertEqual(set(requester.__domains), {"prod.ghe.local"})

        for allowed in [
            "https://prod.ghe.local/github-api/request",
            "https://uploads.prod.ghe.local/path",
            "https://status.prod.ghe.local/path",
        ]:
            requester.__assertUrlAllowed(allowed)

        for not_allowed, arg in [
            ("https://prod.ghe.local/path", "/path"),
            ("https://ghe.local/path", "ghe.local"),
            ("https://api.github.com/request", "api.github.com"),
            ("https://github.com/path", "github.com"),
            ("https://uploads.github.com/path", "uploads.github.com"),
            ("https://status.github.com/path", "status.github.com"),
            ("https://githubusercontent.com/path", "githubusercontent.com"),
            ("https://objects.githubusercontent.com/path", "objects.githubusercontent.com"),
            (
                "https://release-assets.githubusercontent.com/path",
                "release-assets.githubusercontent.com",
            ),
            ("https://example.com/", "example.com"),
        ]:
            with self.assertRaises(AssertionError) as exc:
                requester.__assertUrlAllowed(not_allowed)
            self.assertEqual(exc.exception.args, (arg,))

        # custom (Enterprise) requester with api subdomain and prefix
        requester = github.Github(base_url="https://api.prod.ghe.local/github-api/").requester
        self.assertEqual(set(requester.__domains), {"api.prod.ghe.local", "prod.ghe.local"})

        for allowed in [
            "https://api.prod.ghe.local/github-api/request",
            "https://prod.ghe.local/path",
            "https://uploads.prod.ghe.local/path",
            "https://status.prod.ghe.local/path",
        ]:
            requester.__assertUrlAllowed(allowed)

        for not_allowed, arg in [
            ("https://api.prod.ghe.local/path", "/path"),
            ("https://ghe.local/path", "ghe.local"),
            ("https://api.github.com/request", "api.github.com"),
            ("https://github.com/path", "github.com"),
            ("https://uploads.github.com/path", "uploads.github.com"),
            ("https://status.github.com/path", "status.github.com"),
            ("https://githubusercontent.com/path", "githubusercontent.com"),
            ("https://objects.githubusercontent.com/path", "objects.githubusercontent.com"),
            (
                "https://release-assets.githubusercontent.com/path",
                "release-assets.githubusercontent.com",
            ),
            ("https://example.com/", "example.com"),
        ]:
            with self.assertRaises(AssertionError) as exc:
                requester.__assertUrlAllowed(not_allowed)
            self.assertEqual(exc.exception.args, (arg,))

    def testMakeAbsoluteUrl(self):
        # default github.com requester
        requester = self.g.requester
        assert "/api/v3/request", requester.__makeAbsoluteUrl("/request")
        assert "/api/v3/request", requester.__makeAbsoluteUrl("/request?param=value")
        assert "/api/v3/request", requester.__makeAbsoluteUrl("https://github.com/api/v3/request")
        assert "/api/v3/request", requester.__makeAbsoluteUrl("https://github.com/api/v3/request?param=value")
        assert "/request", requester.__makeAbsoluteUrl("https://github.com/request?param=value")

        # custom (Enterprise) requester with different prefix
        requester = github.Github(base_url="https://api.enterprise.ghe.com/github-api/").requester
        assert "/github-api/request", requester.__makeAbsoluteUrl("/request")
        assert "/github-api/request", requester.__makeAbsoluteUrl("/request?param=value")
        assert "/github-api/request", requester.__makeAbsoluteUrl("https://api.enterprise.ghe.com/github-api/request")
        assert "/github-api/request", requester.__makeAbsoluteUrl(
            "https://api.enterprise.ghe.com/github-api/request?param=value"
        )
        assert "/request", requester.__makeAbsoluteUrl("https://github.com/request?param=value")

    PrimaryRateLimitErrors = [
        "API rate limit exceeded for x.x.x.x. (But here's the good news: Authenticated requests get a higher rate limit. Check out the documentation for more details.)",
    ]
    SecondaryRateLimitErrors = [
        "You have triggered an abuse detection mechanism. Please wait a few minutes before you try again.",
        "You have triggered an abuse detection mechanism and have been temporarily blocked from content creation. Please retry your request again later."
        "You have exceeded a secondary rate limit and have been temporarily blocked from content creation. Please retry your request again later.",
        "You have exceeded a secondary rate limit. Please wait a few minutes before you try again.",
        "Something else here. Please wait a few minutes before you try again.",
    ]
    OtherErrors = ["User does not exist or is not a member of the organization"]

    def testIsRateLimitError(self):
        for message in self.PrimaryRateLimitErrors + self.SecondaryRateLimitErrors:
            self.assertTrue(github.Requester.Requester.isRateLimitError(message), message)
        for message in self.OtherErrors:
            self.assertFalse(github.Requester.Requester.isRateLimitError(message), message)

    def testIsPrimaryRateLimitError(self):
        for message in self.PrimaryRateLimitErrors:
            self.assertTrue(github.Requester.Requester.isPrimaryRateLimitError(message), message)
        for message in self.OtherErrors + self.SecondaryRateLimitErrors:
            self.assertFalse(github.Requester.Requester.isPrimaryRateLimitError(message), message)

    def testIsSecondaryRateLimitError(self):
        for message in self.SecondaryRateLimitErrors:
            self.assertTrue(github.Requester.Requester.isSecondaryRateLimitError(message), message)
        for message in self.OtherErrors + self.PrimaryRateLimitErrors:
            self.assertFalse(github.Requester.Requester.isSecondaryRateLimitError(message), message)

    def assertException(self, exception, exception_type, message, status, data, headers, string):
        self.assertIsInstance(exception, exception_type)
        if message is None:
            self.assertIsNone(exception.message)
        else:
            self.assertEqual(exception.message, message)
        self.assertEqual(exception.status, status)
        if data is None:
            self.assertIsNone(exception.data)
        else:
            self.assertEqual(exception.data, data)
        self.assertEqual(exception.headers, headers)
        self.assertEqual(str(exception), string)

    def testShouldCreateBadCredentialsException(self):
        exc = self.g._Github__requester.createException(401, {"header": "value"}, {"message": "Bad credentials"})
        self.assertException(
            exc,
            github.BadCredentialsException,
            None,
            401,
            {"message": "Bad credentials"},
            {"header": "value"},
            '401 {"message": "Bad credentials"}',
        )

    def testShouldCreateTwoFactorException(self):
        exc = self.g._Github__requester.createException(
            401,
            {"x-github-otp": "required; app"},
            {
                "message": "Must specify two-factor authentication OTP code.",
                "documentation_url": "https://developer.github.com/v3/auth#working-with-two-factor-authentication",
            },
        )
        self.assertException(
            exc,
            github.TwoFactorException,
            None,
            401,
            {
                "message": "Must specify two-factor authentication OTP code.",
                "documentation_url": "https://developer.github.com/v3/auth#working-with-two-factor-authentication",
            },
            {"x-github-otp": "required; app"},
            '401 {"message": "Must specify two-factor authentication OTP code.", "documentation_url": "https://developer.github.com/v3/auth#working-with-two-factor-authentication"}',
        )

    def testShouldCreateBadUserAgentException(self):
        exc = self.g._Github__requester.createException(
            403,
            {"header": "value"},
            {"message": "Missing or invalid User Agent string"},
        )
        self.assertException(
            exc,
            github.BadUserAgentException,
            None,
            403,
            {"message": "Missing or invalid User Agent string"},
            {"header": "value"},
            '403 {"message": "Missing or invalid User Agent string"}',
        )

    def testShouldCreateRateLimitExceededException(self):
        for message in self.PrimaryRateLimitErrors + self.SecondaryRateLimitErrors:
            with self.subTest(message=message):
                exc = self.g._Github__requester.createException(403, {"header": "value"}, {"message": message})
                self.assertException(
                    exc,
                    github.RateLimitExceededException,
                    None,
                    403,
                    {"message": message},
                    {"header": "value"},
                    f'403 {{"message": "{message}"}}',
                )

    def testShouldCreateUnknownObjectException(self):
        exc = self.g._Github__requester.createException(404, {"header": "value"}, {"message": "Not Found"})
        self.assertException(
            exc,
            github.UnknownObjectException,
            None,
            404,
            {"message": "Not Found"},
            {"header": "value"},
            '404 {"message": "Not Found"}',
        )

    def testShouldCreateUnknownObjectException2(self):
        exc = self.g._Github__requester.createException(
            404, {"header": "value"}, {"message": "No object found for the path some-nonexistent-file"}
        )
        self.assertException(
            exc,
            github.UnknownObjectException,
            "No object found for the path some-nonexistent-file",
            404,
            {"message": "No object found for the path some-nonexistent-file"},
            {"header": "value"},
            'No object found for the path some-nonexistent-file: 404 {"message": "No object found for the path some-nonexistent-file"}',
        )

    def testShouldCreateUnknownObjectException3(self):
        exc = self.g._Github__requester.createException(404, {"header": "value"}, {"message": "Branch Not Found"})
        self.assertException(
            exc,
            github.UnknownObjectException,
            "Branch Not Found",
            404,
            {"message": "Branch Not Found"},
            {"header": "value"},
            'Branch Not Found: 404 {"message": "Branch Not Found"}',
        )

    def testShouldCreateGithubException(self):
        for status in range(400, 600):
            with self.subTest(status=status):
                exc = self.g._Github__requester.createException(
                    status, {"header": "value"}, {"message": "Something unknown"}
                )
                self.assertException(
                    exc,
                    github.GithubException,
                    "Something unknown",
                    status,
                    {"message": "Something unknown"},
                    {"header": "value"},
                    f'Something unknown: {status} {{"message": "Something unknown"}}',
                )

    def testShouldCreateExceptionWithoutMessage(self):
        for status in range(400, 600):
            with self.subTest(status=status):
                exc = self.g._Github__requester.createException(status, {}, {})
                self.assertException(exc, github.GithubException, None, status, {}, {}, f"{status} {{}}")

    def testShouldCreateExceptionWithoutOutput(self):
        for status in range(400, 600):
            with self.subTest(status=status):
                exc = self.g._Github__requester.createException(status, {}, None)
                self.assertException(exc, github.GithubException, None, status, None, {}, f"{status}")


class RequesterThrottleTestCase(Framework.TestCase):
    def setUp(self):
        self.setPerPage(10)
        super().setUp()

    mock_time = [datetime.now(timezone.utc)]

    def sleep(self, seconds):
        self.mock_time[0] = self.mock_time[0] + timedelta(seconds=seconds)

    def now(self, tz=None):
        return self.mock_time[0]

    @contextlib.contextmanager
    def mock_sleep(self):
        with mock.patch("github.Requester.time.sleep", side_effect=self.sleep) as sleep_mock, mock.patch(
            "github.Requester.datetime"
        ) as datetime_mock:
            datetime_mock.now = self.now
            yield sleep_mock


class RequesterUnThrottled(RequesterThrottleTestCase):
    def testShouldNotDeferRequests(self):
        with self.mock_sleep() as sleep_mock:
            # same test setup as in RequesterThrottled.testShouldDeferRequests
            with self.replayData("RequesterThrottleTestCase.testDeferRequests.txt"):
                repository = self.g.get_repo(REPO_NAME)
                releases = list(repository.get_releases())
                self.assertEqual(len(releases), 30)

        sleep_mock.assert_not_called()


class RequesterThrottled(RequesterThrottleTestCase):
    def setUp(self):
        self.setSecondsBetweenRequests(1.0)
        self.setSecondsBetweenWrites(3.0)
        super().setUp()

    def testShouldDeferRequests(self):
        with self.mock_sleep() as sleep_mock:
            with self.replayData("RequesterThrottleTestCase.testDeferRequests.txt"):
                # same test setup as in RequesterUnThrottled.testShouldNotDeferRequests
                repository = self.g.get_repo(REPO_NAME)
                releases = [release for release in repository.get_releases()]
                self.assertEqual(len(releases), 30)

        self.assertEqual(sleep_mock.call_args_list, [mock.call(1), mock.call(1), mock.call(1)])

    def testShouldDeferWrites(self):
        with self.mock_sleep() as sleep_mock:
            # same test setup as in AuthenticatedUser.testEmail
            user = self.g.get_user()
            emails = user.get_emails()
            self.assertEqual(
                [item.email for item in emails],
                ["vincent@vincent-jacques.net", "github.com@vincent-jacques.net"],
            )
            self.assertTrue(emails[0].primary)
            self.assertTrue(emails[0].verified)
            self.assertEqual(emails[0].visibility, "private")
            user.add_to_emails("1@foobar.com", "2@foobar.com")
            self.assertEqual(
                [item.email for item in user.get_emails()],
                [
                    "vincent@vincent-jacques.net",
                    "1@foobar.com",
                    "2@foobar.com",
                    "github.com@vincent-jacques.net",
                ],
            )
            user.remove_from_emails("1@foobar.com", "2@foobar.com")
            self.assertEqual(
                [item.email for item in user.get_emails()],
                ["vincent@vincent-jacques.net", "github.com@vincent-jacques.net"],
            )

        self.assertEqual(
            sleep_mock.call_args_list,
            [
                # g.get_user() does not call into GitHub API
                # user.get_emails() is the first request so no waiting needed
                # user.add_to_emails is a write request, this is the first write request
                mock.call(1),
                # user.get_emails() is a read request
                mock.call(1),
                # user.remove_from_emails is a write request, it has to be 3 seconds after the last write
                mock.call(2),
                # user.get_emails() is a read request
                mock.call(1),
            ],
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Arda Kuyumcu <kuyumcuarda@gmail.com>                          #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Denis Blanchette <dblanchette@coveo.com>                      #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 chantra <chantra@users.noreply.github.com>                    #
# Copyright 2024 Bernhard M. Wiedemann <githubbmwprimary@lsmod.de>             #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Jonathan Kliem <jonathan.kliem@gmail.com>                     #
# Copyright 2025 Christoph Reiter <reiter.christoph@gmail.com>                 #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Neel Malik <41765022+neel-m@users.noreply.github.com>         #
# Copyright 2025 Soubhik Kumar Mitra <59209034+x612skm@users.noreply.github.com>#
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import os
from datetime import datetime, timezone
from tempfile import NamedTemporaryFile
from unittest import mock
from unittest.mock import Mock

import jwt

import github
from github.Auth import Auth, Login

from . import Framework
from .GithubIntegration import APP_ID, PRIVATE_KEY, PUBLIC_KEY


class Authentication(Framework.BasicTestCase):
    def testNoAuthentication(self):
        g = github.Github()
        self.assertEqual(g.get_user("jacquev6").name, "Vincent Jacques")

    def testBasicAuthentication(self):
        with self.assertWarns(DeprecationWarning) as warning:
            g = github.Github("login", "password")
        self.assertEqual(g.get_user("jacquev6").name, "Vincent Jacques")
        self.assertWarning(
            warning,
            "Arguments login_or_token and password are deprecated, please use auth=github.Auth.Login(...) instead",
        )

    def testOAuthAuthentication(self):
        with self.assertWarns(DeprecationWarning) as warning:
            g = github.Github(self.oauth_token.token)
        self.assertEqual(g.get_user("jacquev6").name, "Vincent Jacques")
        self.assertWarning(
            warning,
            "Argument login_or_token is deprecated, please use auth=github.Auth.Token(...) instead",
        )

    def testJWTAuthentication(self):
        with self.assertWarns(DeprecationWarning) as warning:
            g = github.Github(jwt=self.jwt.token)
        self.assertEqual(g.get_user("jacquev6").name, "Vincent Jacques")
        self.assertWarning(
            warning,
            "Argument jwt is deprecated, please use auth=github.Auth.AppAuth(...) or "
            "auth=github.Auth.AppAuthToken(...) instead",
        )

    def testAppAuthentication(self):
        with self.assertWarns(DeprecationWarning) as warning:
            app_auth = github.AppAuthentication(
                app_id=APP_ID,
                private_key=PRIVATE_KEY,
                installation_id=29782936,
            )
            g = github.Github(app_auth=app_auth)
        self.assertEqual(g.get_user("ammarmallik").name, "Ammar Akbar")
        self.assertWarnings(
            warning,
            "Use github.Auth.AppInstallationAuth instead",
            "Argument app_auth is deprecated, please use auth=github.Auth.AppInstallationAuth(...) instead",
        )

    def testLoginAuthentication(self):
        # test data copied from testBasicAuthentication to test parity
        g = github.Github(auth=Login("login", "password"))
        self.assertEqual(g.get_user("jacquev6").name, "Vincent Jacques")

    def testTokenAuthentication(self):
        # test data copied from testOAuthAuthentication to test parity
        g = github.Github(auth=self.oauth_token)
        self.assertEqual(g.get_user("jacquev6").name, "Vincent Jacques")

    def testAppAuthTokenAuthentication(self):
        # test data copied from testJWTAuthentication to test parity
        g = github.Github(auth=self.jwt)
        self.assertEqual(g.get_user("jacquev6").name, "Vincent Jacques")

    def testAppAuthAuthentication(self):
        # test data copied from testAppAuthentication to test parity
        g = github.Github(auth=self.app_auth.get_installation_auth(29782936))
        self.assertEqual(g.get_user("ammarmallik").name, "Ammar Akbar")

    def assert_requester_args(self, g, expected_requester):
        expected_args = expected_requester.kwargs
        expected_args.pop("auth")

        auth_args = g._Github__requester.auth.requester.kwargs
        auth_args.pop("auth")

        self.assertEqual(expected_args, auth_args)

        auth_integration_args = (
            g._Github__requester.auth._AppInstallationAuth__integration._GithubIntegration__requester.kwargs
        )
        auth_integration_args.pop("auth")

        self.assertEqual(expected_args, auth_integration_args)

    def testAppAuthAuthenticationWithGithubRequesterArgs(self):
        # test that Requester arguments given to github.Github are passed to auth and auth.__integration
        g = github.Github(
            auth=self.app_auth.get_installation_auth(29782936),
            base_url="https://base.net/",
            timeout=60,
            user_agent="agent",
            per_page=100,
            verify="cert",
            retry=999,
            pool_size=10,
            seconds_between_requests=100,
            seconds_between_writes=1000,
        )

        self.assert_requester_args(g, g._Github__requester)

    def testAppAuthAuthenticationWithGithubIntegrationRequesterArgs(self):
        # test that Requester arguments given to github.GithubIntegration are passed to auth and auth.__integration
        gi = github.GithubIntegration(
            auth=self.app_auth,
            base_url="https://base.net/",
            timeout=60,
            user_agent="agent",
            per_page=100,
            verify="cert",
            retry=999,
            pool_size=10,
            seconds_between_requests=100,
            seconds_between_writes=1000,
        )

        self.assert_requester_args(gi.get_github_for_installation(29782936), gi._GithubIntegration__requester)

    def testAppInstallationAuthAuthentication(self):
        # test data copied from testAppAuthentication to test parity
        installation_auth = github.Auth.AppInstallationAuth(self.app_auth, 29782936)
        g = github.Github(
            auth=installation_auth, lazy=False, seconds_between_requests=None, seconds_between_writes=None
        )

        # token expires 2024-11-25 01:00:02
        token = installation_auth.token
        self.assertEqual(
            installation_auth._AppInstallationAuth__installation_authorization.expires_at,
            datetime(2024, 11, 25, 1, 0, 2, tzinfo=timezone.utc),
        )

        # test token expiry
        # control the current time used by _is_expired
        with mock.patch("github.Auth.datetime") as dt:
            # just before expiry
            dt.now = mock.Mock(return_value=datetime(2024, 11, 25, 0, 59, 3, tzinfo=timezone.utc))
            self.assertFalse(installation_auth._is_expired)

            # just after expiry
            dt.now = mock.Mock(return_value=datetime(2024, 11, 25, 1, 0, 3, tzinfo=timezone.utc))
            self.assertTrue(installation_auth._is_expired)

            # expect refreshing the token
            refreshed_token = installation_auth.token
            self.assertNotEqual(refreshed_token, token)
            self.assertFalse(installation_auth._is_expired)
            self.assertEqual(
                installation_auth._AppInstallationAuth__installation_authorization.expires_at,
                datetime(2025, 11, 25, 1, 0, 2, tzinfo=timezone.utc),
            )

            # use the token
            self.assertEqual(g.get_user("ammarmallik").name, "Ammar Akbar")
            self.assertEqual(g.get_repo("PyGithub/PyGithub").full_name, "PyGithub/PyGithub")

    def testAppInstallationAuthAuthenticationRequesterArgs(self):
        installation_auth = github.Auth.AppInstallationAuth(self.app_auth, 29782936)
        github.Github(
            auth=installation_auth,
        )

    def testAppUserAuthentication(self):
        client_id = "removed client id"
        client_secret = "removed client secret"
        refresh_token = "removed refresh token"

        g = github.Github()
        app = g.get_oauth_application(client_id, client_secret)
        with mock.patch("github.AccessToken.datetime") as dt:
            dt.now = mock.Mock(return_value=datetime(2023, 6, 7, 12, 0, 0, 123, tzinfo=timezone.utc))
            token = app.refresh_access_token(refresh_token)
        self.assertEqual(token.token, "fresh access token")
        self.assertEqual(token.type, "bearer")
        self.assertEqual(token.scope, "")
        self.assertEqual(token.expires_in, 28800)
        self.assertEqual(
            token.expires_at,
            datetime(2023, 6, 7, 20, 0, 0, 123, tzinfo=timezone.utc),
        )
        self.assertEqual(token.refresh_token, "fresh refresh token")
        self.assertEqual(token.refresh_expires_in, 15811200)
        self.assertEqual(
            token.refresh_expires_at,
            datetime(2023, 12, 7, 12, 0, 0, 123, tzinfo=timezone.utc),
        )

        auth = app.get_app_user_auth(token)
        with mock.patch("github.Auth.datetime") as dt:
            dt.now = mock.Mock(return_value=datetime(2023, 6, 7, 20, 0, 0, 123, tzinfo=timezone.utc))
            self.assertEqual(auth._is_expired, False)
            self.assertEqual(auth.token, "fresh access token")
        self.assertEqual(auth.token_type, "bearer")
        self.assertEqual(auth.refresh_token, "fresh refresh token")

        # expire auth token
        with mock.patch("github.Auth.datetime") as dt:
            dt.now = mock.Mock(return_value=datetime(2023, 6, 7, 20, 0, 1, 123, tzinfo=timezone.utc))
            self.assertEqual(auth._is_expired, True)
            self.assertEqual(auth.token, "another access token")
            self.assertEqual(auth._is_expired, False)
        self.assertEqual(auth.token_type, "bearer")
        self.assertEqual(auth.refresh_token, "another refresh token")

        g = github.Github(auth=auth)
        user = g.get_user()
        self.assertEqual(user.login, "EnricoMi")

    def testNetrcAuth(self):
        with NamedTemporaryFile("wt", delete=False) as tmp:
            # write temporary netrc file
            tmp.write("machine api.github.com\n")
            tmp.write("login github-user\n")
            tmp.write("password github-password\n")
            tmp.close()

            auth = github.Auth.NetrcAuth()
            with mock.patch.dict(os.environ, {"NETRC": tmp.name}):
                github.Github(auth=auth)

            self.assertEqual(auth.login, "github-user")
            self.assertEqual(auth.password, "github-password")
            self.assertEqual(auth.token, "Z2l0aHViLXVzZXI6Z2l0aHViLXBhc3N3b3Jk")
            self.assertEqual(auth.token_type, "Basic")

    def testNetrcAuthFails(self):
        # provide an empty netrc file to make sure this test does not find one
        with NamedTemporaryFile("wt", delete=False) as tmp:
            tmp.close()
            auth = github.Auth.NetrcAuth()
            with mock.patch.dict(os.environ, {"NETRC": tmp.name}):
                with self.assertRaises(RuntimeError) as exc:
                    github.Github(auth=auth)
                self.assertEqual(exc.exception.args, ("Could not get credentials from netrc for host api.github.com",))

    def testCreateJWT(self):
        auth = github.Auth.AppAuth(APP_ID, PRIVATE_KEY)

        with mock.patch("github.Auth.time") as t:
            t.time = mock.Mock(return_value=1550055331.7435968)
            token = auth.create_jwt()

        payload = jwt.decode(
            token,
            key=PUBLIC_KEY,
            algorithms=["RS256"],
            options={"verify_exp": False},
            issuer=str(APP_ID),
        )
        self.assertDictEqual(payload, {"iat": 1550055271, "exp": 1550055631, "iss": str(APP_ID)})

    def testCreateJWTWithExpiration(self):
        auth = github.Auth.AppAuth(APP_ID, PRIVATE_KEY, jwt_expiry=120, jwt_issued_at=-30)

        with mock.patch("github.Auth.time") as t:
            t.time = mock.Mock(return_value=1550055331.7435968)
            token = auth.create_jwt(60)

        payload = jwt.decode(
            token,
            key=PUBLIC_KEY,
            algorithms=["RS256"],
            options={"verify_exp": False},
            issuer=str(APP_ID),
        )
        self.assertDictEqual(payload, {"iat": 1550055301, "exp": 1550055391, "iss": str(APP_ID)})

    def testUserAgent(self):
        g = github.Github(user_agent="PyGithubTester")
        self.assertEqual(g.get_user("jacquev6").name, "Vincent Jacques")

    def testAuthorizationHeaderWithLogin(self):
        # See special case in Framework.fixAuthorizationHeader
        g = github.Github(auth=github.Auth.Login("fake_login", "fake_password"))
        with self.assertRaises(github.GithubException):
            g.get_user().name

    def testAuthorizationHeaderWithToken(self):
        # See special case in Framework.fixAuthorizationHeader
        g = github.Github(auth=github.Auth.Token("ZmFrZV9sb2dpbjpmYWtlX3Bhc3N3b3Jk"))
        with self.assertRaises(github.GithubException):
            g.get_user().name

    def testAddingCustomHeaders(self):
        requester = github.Github(auth=CustomAuth())._Github__requester

        def requestRaw(cnx, verb, url, requestHeaders, encoded_input, stream=False, follow_302_redirect=False):
            self.modifiedHeaders = requestHeaders
            return Mock(), {}, Mock()

        requester._Requester__requestRaw = requestRaw
        requestHeaders = {"Custom key": "secret"}
        requester._Requester__requestEncode(None, "GET", "http://github.com", None, requestHeaders, None, Mock())

        self.assertEqual("Custom token", self.modifiedHeaders["Custom key"])


class CustomAuth(Auth):
    @property
    def token_type(self) -> str:
        return "custom auth"

    @property
    def token(self) -> str:
        return "Custom token"

    def authentication(self, headers):
        headers["Custom key"] = self.token

    def mask_authentication(self, headers):
        headers["Custom key"] = "Masked custom header"



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2016 humbug <bah>                                                  #
# Copyright 2017 Hugo <hugovk@users.noreply.github.com>                        #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 Tuuu Nya <yuzesheji@qq.com>                                   #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import pickle
from unittest import mock

import github

from . import Framework


class Exceptions(Framework.TestCase):
    def testInvalidInput(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            self.g.get_user().create_key("Bad key", "xxx")
        self.assertIsInstance(raisedexp.exception, github.GithubException)
        self.assertEqual(raisedexp.exception.message, "Validation Failed")
        self.assertEqual(raisedexp.exception.status, 422)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "errors": [
                    {
                        "code": "custom",
                        "field": "key",
                        "message": "key is invalid. It must begin with 'ssh-rsa' or 'ssh-dss'. Check that you're copying the public half of the key",
                        "resource": "PublicKey",
                    }
                ],
                "message": "Validation Failed",
            },
        )

    def testNonJsonDataReturnedByGithub(self):
        # Replay data was forged according to https://github.com/jacquev6/PyGithub/pull/182
        with self.assertRaises(github.GithubException) as raisedexp:
            # 503 would be retried, disable retries
            self.get_github(self.authMode, retry=None).get_user("jacquev6")
        self.assertIsInstance(raisedexp.exception, github.GithubException)
        self.assertIsNone(raisedexp.exception.message)
        self.assertEqual(raisedexp.exception.status, 503)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "data": "<html><body><h1>503 Service Unavailable</h1>No server is available to handle this request.</body></html>",
            },
        )

    def testUnknownObject(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            self.g.get_user().get_repo("Xxx")
        self.assertIsInstance(raisedexp.exception, github.UnknownObjectException)
        self.assertIsNone(raisedexp.exception.message)
        self.assertEqual(raisedexp.exception.status, 404)
        self.assertEqual(raisedexp.exception.data, {"message": "Not Found"})
        self.assertEqual(str(raisedexp.exception), '404 {"message": "Not Found"}')

    def testUnknownUser(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            self.g.get_user("ThisUserShouldReallyNotExist")
        self.assertIsInstance(raisedexp.exception, github.UnknownObjectException)
        self.assertIsNone(raisedexp.exception.message)
        self.assertEqual(raisedexp.exception.status, 404)
        self.assertEqual(raisedexp.exception.data, {"message": "Not Found"})
        self.assertEqual(str(raisedexp.exception), '404 {"message": "Not Found"}')

    def testBadAuthentication(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            github.Github(auth=github.Auth.Login("BadUser", "BadPassword")).get_user().login
        self.assertIsInstance(raisedexp.exception, github.BadCredentialsException)
        self.assertIsNone(raisedexp.exception.message)
        self.assertEqual(raisedexp.exception.status, 401)
        self.assertEqual(raisedexp.exception.data, {"message": "Bad credentials"})
        self.assertEqual(str(raisedexp.exception), '401 {"message": "Bad credentials"}')

    def testExceptionPickling(self):
        pickle.loads(pickle.dumps(github.GithubException("foo", "bar", None)))

    def testJSONParseError(self):
        # Replay data was forged to force a JSON error
        with self.assertRaises(ValueError):
            self.g.get_user("jacquev6")


class SpecificExceptions(Framework.TestCase):
    def testBadCredentials(self):
        self.assertRaises(
            github.BadCredentialsException,
            lambda: github.Github(auth=github.Auth.Login("BadUser", "BadPassword")).get_user().login,
        )

    def test2FARequired(self):
        self.assertRaises(
            github.TwoFactorException,
            lambda: github.Github(auth=github.Auth.Login("2fauser", "password")).get_user().login,
        )

    def testUnknownObject(self):
        self.assertRaises(github.UnknownObjectException, lambda: self.g.get_user().get_repo("Xxx"))

    def testBadUserAgent(self):
        self.assertRaises(
            github.BadUserAgentException,
            lambda: github.Github(auth=self.oauth_token, user_agent="").get_user().name,
        )

    def testRateLimitExceeded(self):
        # rate limit errors would be retried if retry is not set None
        g = github.Github(retry=None)

        def exceed():
            for i in range(100):
                g.get_user("jacquev6")

        self.assertRaises(github.RateLimitExceededException, exceed)

    def testAuthenticatedRateLimitExceeded(self):
        def exceed():
            for i in range(100):
                res = self.g.search_code("jacquev6")
                res.get_page(0)

        with self.assertRaises(github.RateLimitExceededException) as raised:
            exceed()
        self.assertEqual(raised.exception.headers.get("retry-after"), "60")

    def testIncompletableObject(self):
        github.UserKey.UserKey.setCheckAfterInitFlag(False)
        obj = github.UserKey.UserKey(mock.MagicMock(), {}, {}, False)
        self.assertRaises(github.IncompletableObject, obj._completeIfNeeded)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2021 Claire Johns <42869556+johnsc1@users.noreply.github.com>      #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class IssueDependenciesSummary(Framework.TestCase):
    def setUp(self):
        super().setUp()
        with self.replayData("Issue.setUp.txt"):
            self.repo = self.g.get_repo("PyGithub/PyGithub")
            self.issue = self.repo.get_issue(28)
            self.ids = self.issue.issue_dependencies_summary

    def testAttributes(self):
        self.assertEqual(self.ids.blocked_by, 0)
        self.assertEqual(self.ids.blocking, 0)
        self.assertEqual(self.ids.total_blocked_by, 1)
        self.assertEqual(self.ids.total_blocking, 1)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2018 Justin Kufro <jkufro@andrew.cmu.edu>                          #
# Copyright 2019 Isac Souza <isouza@daitan.com>                                #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Liuyang Wan <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 Amador Pahim <apahim@redhat.com>                              #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Maja Massarini <2678400+majamassarini@users.noreply.github.com>#
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import requests
import responses
import urllib3

import github

from . import Framework

REPO_NAME = "PyGithub/PyGithub"


class Retry(Framework.TestCase):
    def setUp(self):
        # status codes returned on random github server errors
        status_forcelist = (500, 502, 504)
        retry = urllib3.Retry(total=3, read=3, connect=3, status_forcelist=status_forcelist)
        self.setRetry(retry)
        super().setUp()

    def testShouldNotRetryWhenStatusNotOnList(self):
        with self.assertRaises(github.GithubException):
            self.g.get_repo(REPO_NAME)
        self.assertEqual(len(responses.calls), 1)

    def testReturnsRepoAfter3Retries(self):
        repository = self.g.get_repo(REPO_NAME)
        self.assertEqual(len(responses.calls), 4)
        for call in responses.calls:
            self.assertEqual(call.request.path_url, "/repos/" + REPO_NAME)

        self.assertIsInstance(repository, github.Repository.Repository)
        self.assertEqual(repository.full_name, REPO_NAME)

    def testReturnsRepoAfter1Retry(self):
        repository = self.g.get_repo(REPO_NAME)
        self.assertEqual(len(responses.calls), 2)
        for call in responses.calls:
            self.assertEqual(call.request.path_url, "/repos/" + REPO_NAME)

        self.assertIsInstance(repository, github.Repository.Repository)
        self.assertEqual(repository.full_name, REPO_NAME)

    def testRaisesRetryErrorAfterMaxRetries(self):
        with self.assertRaises(requests.exceptions.RetryError):
            self.g.get_repo("PyGithub/PyGithub")
        self.assertEqual(len(responses.calls), 4)
        for call in responses.calls:
            self.assertEqual(call.request.path_url, "/repos/PyGithub/PyGithub")

    def testReturnsRepoAfterSettingRetryHttp(self):
        g = github.Github(
            auth=self.oauth_token,
            base_url="http://my.enterprise.com",
            retry=0,
        )  # http here
        repository = g.get_repo(REPO_NAME)
        self.assertIsInstance(repository, github.Repository.Repository)
        self.assertEqual(repository.full_name, REPO_NAME)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2020 Dhruv Manilawala <dhruvmanila@gmail.com>                      #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class CheckRun(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("PyGithub/PyGithub")
        self.testrepo = self.g.get_repo("dhruvmanila/pygithub-testing")
        self.check_run_id = 34942661139
        self.check_run_ref = "10a7135a04f71e6101f8b013aded8a662d08fd1f"
        self.check_run = self.repo.get_check_run(self.check_run_id)
        self.commit = self.repo.get_commit(self.check_run_ref)

    def testAttributes(self):
        self.assertEqual(self.check_run.app.id, 15368)
        self.assertEqual(self.check_run.app.slug, "github-actions")
        self.assertEqual(self.check_run.check_suite.id, 32504127411)
        self.assertEqual(
            self.check_run.check_suite.url, "https://api.github.com/repos/PyGithub/PyGithub/check-suites/32504127411"
        )
        # check_suite is lazy, so accessing a property other than id or url fetches the suite object
        self.assertEqual(self.check_run.check_suite.head_sha, "10a7135a04f71e6101f8b013aded8a662d08fd1f")
        self.assertEqual(self.check_run.check_suite_id, 32504127411)
        self.assertEqual(self.check_run.completed_at, datetime(2024, 12, 28, 16, 53, 10, tzinfo=timezone.utc))
        self.assertEqual(self.check_run.conclusion, "success")
        self.assertIsNone(self.check_run.deployment)
        self.assertEqual(
            self.check_run.details_url, "https://github.com/PyGithub/PyGithub/actions/runs/12528252236/job/34942661139"
        )
        self.assertEqual(self.check_run.external_id, "8ece7711-e8e8-5d87-8f8a-6791d424ecd6")
        self.assertEqual(self.check_run.head_sha, "10a7135a04f71e6101f8b013aded8a662d08fd1f")
        self.assertEqual(
            self.check_run.html_url, "https://github.com/PyGithub/PyGithub/actions/runs/12528252236/job/34942661139"
        )
        self.assertEqual(self.check_run.id, 34942661139)
        self.assertEqual(self.check_run.name, "test (Python 3.8 on Ubuntu)")
        self.assertEqual(self.check_run.node_id, "CR_kwDOADYVqs8AAAAIIr6yEw")
        self.assertEqual(self.check_run.output.annotations_count, 1)
        self.assertEqual(len(self.check_run.pull_requests), 0)
        self.assertEqual(self.check_run.started_at, datetime(2024, 12, 28, 16, 51, 59, tzinfo=timezone.utc))
        self.assertEqual(self.check_run.status, "completed")
        self.assertEqual(self.check_run.url, "https://api.github.com/repos/PyGithub/PyGithub/check-runs/34942661139")
        self.assertEqual(repr(self.check_run), 'CheckRun(id=34942661139, conclusion="success")')

    def testLazyAttributes(self):
        run = self.g.withLazy(True).get_repo("lazy/repo").get_check_run(42)
        self.assertEqual(str(run), "CheckRun(id=42, conclusion=None)")
        self.assertEqual(run.id, 42)
        self.assertEqual(run.url, "/repos/lazy/repo/check-runs/42")

    def testCheckRunOutputAttributes(self):
        check_run_output = self.repo.get_check_run(1039891917).output
        self.assertEqual(check_run_output.title, "test (Python 3.6)")
        self.assertEqual(
            check_run_output.summary,
            "There are 1 failures, 0 warnings, and 0 notices.",
        )
        self.assertIsNone(check_run_output.text)
        self.assertEqual(check_run_output.annotations_count, 1)
        self.assertEqual(
            check_run_output.annotations_url,
            "https://api.github.com/repos/PyGithub/PyGithub/check-runs/1039891917/annotations",
        )
        self.assertEqual(repr(check_run_output), 'CheckRunOutput(title="test (Python 3.6)")')

    def testGetCheckRunsForRef(self):
        check_runs = self.commit.get_check_runs()
        self.assertEqual(check_runs.totalCount, 4)
        self.assertListEqual(
            [check_run.id for check_run in check_runs],
            [34942661139, 1039891931, 1039891917, 1039891902],
        )

    def testGetCheckRunsForRefFilterByCheckName(self):
        check_runs = self.commit.get_check_runs(check_name="test (Python 3.6)")
        self.assertEqual(check_runs.totalCount, 1)
        self.assertListEqual([check_run.id for check_run in check_runs], [1039891917])

    def testGetCheckRunsForRefFilterByStatus(self):
        completed_check_runs = self.commit.get_check_runs(status="completed")
        self.assertEqual(completed_check_runs.totalCount, 4)
        self.assertListEqual(
            [check_run.id for check_run in completed_check_runs],
            [34942661139, 1039891931, 1039891917, 1039891902],
        )
        queued_check_runs = self.commit.get_check_runs(status="queued")
        self.assertEqual(queued_check_runs.totalCount, 0)
        in_progress_check_runs = self.commit.get_check_runs(status="in_progress")
        self.assertEqual(in_progress_check_runs.totalCount, 0)

    def testGetCheckRunsForRefFilterByFilter(self):
        latest_check_runs = self.commit.get_check_runs(filter="latest")
        all_check_runs = self.commit.get_check_runs(filter="all")
        self.assertEqual(latest_check_runs.totalCount, 4)
        self.assertListEqual(
            [check_run.id for check_run in latest_check_runs],
            [34942661139, 1039891931, 1039891917, 1039891902],
        )
        self.assertEqual(all_check_runs.totalCount, 4)
        self.assertListEqual(
            [check_run.id for check_run in all_check_runs],
            [34942661139, 1039891931, 1039891917, 1039891902],
        )

    def testCreateCheckRunInProgress(self):
        check_run = self.testrepo.create_check_run(
            name="basic_check_run",
            head_sha="0283d46537193f1fed7d46859f15c5304b9836f9",
            status="in_progress",
            external_id="50",
            details_url="https://www.example.com",
            started_at=datetime(2020, 9, 4, 1, 14, 52),
            output={"title": "PyGithub Check Run Test", "summary": "Test summary"},
        )
        self.assertEqual(check_run.name, "basic_check_run")
        self.assertEqual(check_run.head_sha, "0283d46537193f1fed7d46859f15c5304b9836f9")
        self.assertEqual(check_run.status, "in_progress")
        self.assertEqual(check_run.external_id, "50")
        self.assertEqual(
            check_run.started_at,
            datetime(2020, 9, 4, 1, 14, 52, tzinfo=timezone.utc),
        )
        self.assertEqual(check_run.output.title, "PyGithub Check Run Test")
        self.assertEqual(check_run.output.summary, "Test summary")
        self.assertIsNone(check_run.output.text)
        self.assertEqual(check_run.output.annotations_count, 0)
        # We don't want to keep this hanging
        check_run.edit(conclusion="success")
        self.assertEqual(check_run.conclusion, "success")
        self.assertEqual(check_run.status, "completed")

    def testCreateCheckRunCompleted(self):
        check_run = self.testrepo.create_check_run(
            name="completed_check_run",
            head_sha="0283d46537193f1fed7d46859f15c5304b9836f9",
            status="completed",
            started_at=datetime(2020, 10, 20, 10, 30, 29),
            conclusion="success",
            completed_at=datetime(2020, 10, 20, 11, 30, 50),
            output={
                "title": "Readme report",
                "summary": "There are 0 failures, 2 warnings, and 1 notices.",
                "text": "You may have some misspelled words on lines 2 and 4.",
                "annotations": [
                    {
                        "path": "README.md",
                        "annotation_level": "warning",
                        "title": "Spell Checker",
                        "message": "Check your spelling for 'banaas'.",
                        "raw_details": "Do you mean 'bananas' or 'banana'?",
                        "start_line": 2,
                        "end_line": 2,
                    },
                    {
                        "path": "README.md",
                        "annotation_level": "warning",
                        "title": "Spell Checker",
                        "message": "Check your spelling for 'aples'",
                        "raw_details": "Do you mean 'apples' or 'Naples'",
                        "start_line": 4,
                        "end_line": 4,
                    },
                ],
                "images": [
                    {
                        "alt": "Test Image",
                        "image_url": "http://example.com/images/42",
                    }
                ],
            },
            actions=[
                {
                    "label": "Fix",
                    "identifier": "fix_errors",
                    "description": "Allow us to fix these errors for you",
                }
            ],
        )
        self.assertEqual(check_run.name, "completed_check_run")
        self.assertEqual(check_run.head_sha, "0283d46537193f1fed7d46859f15c5304b9836f9")
        self.assertEqual(check_run.status, "completed")
        self.assertEqual(
            check_run.started_at,
            datetime(2020, 10, 20, 10, 30, 29, tzinfo=timezone.utc),
        ),
        self.assertEqual(check_run.conclusion, "success")
        self.assertEqual(
            check_run.completed_at,
            datetime(2020, 10, 20, 11, 30, 50, tzinfo=timezone.utc),
        ),
        self.assertEqual(check_run.output.annotations_count, 2)

    def testUpdateCheckRunSuccess(self):
        # This is a different check run created for this test
        check_run = self.testrepo.create_check_run(
            name="edit_check_run",
            head_sha="0283d46537193f1fed7d46859f15c5304b9836f9",
            status="in_progress",
            external_id="100",
            started_at=datetime(2020, 10, 20, 14, 24, 31),
            output={"title": "Check run for testing edit method", "summary": ""},
        )
        self.assertEqual(check_run.name, "edit_check_run")
        self.assertEqual(check_run.status, "in_progress")
        check_run.edit(
            status="completed",
            conclusion="success",
            output={
                "title": "Check run for testing edit method",
                "summary": "This is the summary of editing check run as completed.",
            },
        )
        self.assertEqual(check_run.name, "edit_check_run")
        self.assertEqual(check_run.status, "completed")
        self.assertEqual(check_run.conclusion, "success")
        self.assertEqual(check_run.output.title, "Check run for testing edit method")
        self.assertEqual(
            check_run.output.summary,
            "This is the summary of editing check run as completed.",
        )
        self.assertEqual(check_run.output.annotations_count, 0)

    def testUpdateCheckRunFailure(self):
        # This is a different check run created for this test
        check_run = self.testrepo.create_check_run(
            name="fail_check_run",
            head_sha="0283d46537193f1fed7d46859f15c5304b9836f9",
            status="in_progress",
            external_id="101",
            started_at=datetime(2020, 10, 20, 10, 14, 51),
            output={"title": "Check run for testing failure", "summary": ""},
        )
        self.assertEqual(check_run.name, "fail_check_run")
        self.assertEqual(check_run.status, "in_progress")
        check_run.edit(
            status="completed",
            conclusion="failure",
            output={
                "title": "Check run for testing failure",
                "summary": "There is 1 whitespace error.",
                "text": "You may have a whitespace error in the file 'test.py'",
                "annotations": [
                    {
                        "path": "test.py",
                        "annotation_level": "failure",
                        "title": "whitespace checker",
                        "message": "Remove the unnecessary whitespace from the file.",
                        "start_line": 2,
                        "end_line": 2,
                        "start_column": 17,
                        "end_column": 18,
                    }
                ],
            },
            actions=[
                {
                    "label": "Fix",
                    "identifier": "fix_errors",
                    "description": "Allow us to fix these errors for you",
                }
            ],
        )
        self.assertEqual(check_run.status, "completed")
        self.assertEqual(check_run.conclusion, "failure")
        self.assertEqual(check_run.output.annotations_count, 1)

    def testUpdateCheckRunAll(self):
        check_run = self.testrepo.get_check_run(1279259090)
        check_run.edit(
            name="update_all_params",
            head_sha="0283d46537193f1fed7d46859f15c5304b9836f9",
            details_url="https://www.example-url.com",
            external_id="49",
            started_at=datetime(2020, 10, 20, 1, 10, 20),
            completed_at=datetime(2020, 10, 20, 2, 20, 30),
            actions=[
                {
                    "label": "Hello World!",
                    "identifier": "identity",
                    "description": "Hey! This is a test",
                }
            ],
        )
        self.assertEqual(check_run.name, "update_all_params")
        self.assertEqual(check_run.head_sha, "0283d46537193f1fed7d46859f15c5304b9836f9")
        self.assertEqual(check_run.details_url, "https://www.example-url.com")
        self.assertEqual(check_run.external_id, "49")
        self.assertEqual(
            check_run.started_at,
            datetime(2020, 10, 20, 1, 10, 20, tzinfo=timezone.utc),
        )
        self.assertEqual(
            check_run.completed_at,
            datetime(2020, 10, 20, 2, 20, 30, tzinfo=timezone.utc),
        )

    def testCheckRunAnnotationAttributes(self):
        check_run = self.testrepo.get_check_run(1280914700)
        self.assertEqual(check_run.name, "annotations")
        annotation = check_run.get_annotations()[0]
        self.assertEqual(annotation.annotation_level, "warning")
        self.assertIsNone(annotation.end_column)
        self.assertEqual(annotation.end_line, 2)
        self.assertEqual(annotation.message, "Check your spelling for 'banaas'.")
        self.assertEqual(annotation.path, "README.md")
        self.assertEqual(annotation.raw_details, "Do you mean 'bananas' or 'banana'?")
        self.assertIsNone(annotation.start_column)
        self.assertEqual(annotation.start_line, 2)
        self.assertEqual(annotation.title, "Spell Checker")
        self.assertEqual(
            annotation.blob_href,
            "https://github.com/dhruvmanila/pygithub-testing/blob/0283d46537193f1fed7d46859f15c5304b9836f9/README.md",
        )
        self.assertEqual(repr(annotation), 'CheckRunAnnotation(title="Spell Checker")')

    def testListCheckRunAnnotations(self):
        check_run = self.testrepo.get_check_run(1280914700)
        self.assertEqual(check_run.name, "annotations")
        self.assertEqual(check_run.status, "completed")
        annotation_list = check_run.get_annotations()
        self.assertEqual(annotation_list.totalCount, 2)
        self.assertListEqual([annotation.start_line for annotation in annotation_list], [2, 4])



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class RateLimitOverview(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.rate_limit_overview = self.g.get_rate_limit()

    def testAttributes(self):
        self.assertIsNotNone(self.rate_limit_overview.rate)
        self.assertIsNotNone(self.rate_limit_overview.resources)
        self.assertEqual(self.rate_limit_overview.rate.limit, 5000)
        self.assertEqual(self.rate_limit_overview.rate.remaining, 4988)
        self.assertEqual(self.rate_limit_overview.rate.reset, datetime(2024, 12, 13, 6, 43, 18, tzinfo=timezone.utc))
        self.assertEqual(self.rate_limit_overview.rate.used, 12)

        self.assertEqual(
            str(self.rate_limit_overview.resources.actions_runner_registration),
            "Rate(reset=2024-12-13 07:28:18+00:00, remaining=10000, limit=10000)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.code_scanning_upload),
            "Rate(reset=2024-12-13 07:28:18+00:00, remaining=1000, limit=1000)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.code_search),
            "Rate(reset=2024-12-13 06:29:18+00:00, remaining=10, limit=10)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.core),
            "Rate(reset=2024-12-13 06:43:18+00:00, remaining=4988, limit=5000)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.dependency_snapshots),
            "Rate(reset=2024-12-13 06:29:18+00:00, remaining=100, limit=100)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.graphql),
            "Rate(reset=2024-12-13 06:43:42+00:00, remaining=4808, limit=5000)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.integration_manifest),
            "Rate(reset=2024-12-13 07:28:18+00:00, remaining=5000, limit=5000)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.scim),
            "Rate(reset=2024-12-13 07:28:18+00:00, remaining=15000, limit=15000)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.search),
            "Rate(reset=2024-12-13 06:29:18+00:00, remaining=30, limit=30)",
        )
        self.assertEqual(
            str(self.rate_limit_overview.resources.source_import),
            "Rate(reset=2024-12-13 06:29:18+00:00, remaining=100, limit=100)",
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Jon Dufresne <jon.dufresne@gmail.com>                         #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github

from . import Framework


class Gist(Framework.TestCase):
    def testAttributes(self):
        gist = self.g.get_gist("6296732")
        self.assertEqual(gist.comments, 0)
        self.assertIsNone(gist.comments_enabled)
        self.assertEqual(gist.comments_url, "https://api.github.com/gists/6296732/comments")
        self.assertEqual(gist.commits_url, "https://api.github.com/gists/6296732/commits")
        self.assertEqual(
            gist.created_at,
            datetime(2013, 8, 21, 16, 28, 24, tzinfo=timezone.utc),
        )
        self.assertEqual(gist.description, "Github API")
        self.assertEqual(list(gist.files.keys()), ["GithubAPI.lua"])
        self.assertEqual(gist.files["GithubAPI.lua"].size, 21229)
        self.assertEqual(gist.files["GithubAPI.lua"].filename, "GithubAPI.lua")
        self.assertEqual(gist.files["GithubAPI.lua"].language, "Lua")
        self.assertEqual(gist.files["GithubAPI.lua"].content[:10], "-- GithubA")
        self.assertEqual(
            gist.files["GithubAPI.lua"].raw_url,
            "https://gist.githubusercontent.com/jacquev6/6296732/raw/88aafa25fb28e17013054a117354a37f0d78963c/GithubAPI.lua",
        )
        self.assertEqual(gist.fork_of.id, "6296553")
        self.assertEqual(gist.forks, [])
        self.assertEqual(gist.forks_url, "https://api.github.com/gists/6296732/forks")
        self.assertEqual(gist.git_pull_url, "https://gist.github.com/6296732.git")
        self.assertEqual(gist.git_push_url, "https://gist.github.com/6296732.git")
        self.assertEqual(len(gist.history), 1)
        self.assertEqual(gist.history[0].change_status.additions, 793)
        self.assertEqual(gist.history[0].change_status.deletions, 0)
        self.assertEqual(gist.history[0].change_status.total, 793)
        self.assertEqual(
            gist.history[0].committed_at,
            datetime(2013, 8, 21, 16, 12, 27, tzinfo=timezone.utc),
        )
        self.assertEqual(
            gist.history[0].url,
            "https://api.github.com/gists/6296732/c464aecd7fea16684e935607eeea7ae4f8caa0e2",
        )
        self.assertEqual(gist.history[0].user, None)
        self.assertEqual(gist.history[0].owner.login, "jacquev6")
        self.assertEqual(gist.history[0].version, "c464aecd7fea16684e935607eeea7ae4f8caa0e2")
        self.assertEqual(gist.html_url, "https://gist.github.com/6296732")
        self.assertEqual(gist.id, "6296732")
        self.assertIsNone(gist.node_id)
        self.assertEqual(gist.owner.login, "jacquev6")
        self.assertTrue(gist.public)
        self.assertIsNone(gist.truncated)
        self.assertEqual(
            gist.updated_at,
            datetime(2013, 8, 21, 16, 28, 24, tzinfo=timezone.utc),
        )
        self.assertEqual(gist.url, "https://api.github.com/gists/6296732")
        self.assertEqual(gist.user, None)
        self.assertEqual(gist.owner.login, "jacquev6")
        self.assertEqual(gist.git_pull_url, "https://gist.github.com/6296732.git")
        self.assertEqual(gist.git_push_url, "https://gist.github.com/6296732.git")
        self.assertEqual(gist.html_url, "https://gist.github.com/6296732")
        self.assertEqual(gist.url, "https://api.github.com/gists/6296732")
        self.assertEqual(repr(gist), 'Gist(id="6296732")')
        self.assertEqual(repr(gist.files["GithubAPI.lua"]), 'GistFile(filename="GithubAPI.lua")')

    def testLazyAttributes(self):
        gist = self.g.withLazy(True).get_gist("gist")
        self.assertEqual(str(gist), 'Gist(id="gist")')
        self.assertEqual(gist.id, "gist")
        self.assertEqual(gist.url, "/gists/gist")

    def testEditWithoutParameters(self):
        gist = self.g.get_gist("2729810")
        gist.edit()
        self.assertEqual(gist.description, "Gist created by PyGithub")
        self.assertEqual(
            gist.updated_at,
            datetime(2012, 5, 19, 7, 0, 58, tzinfo=timezone.utc),
        )

    def testEditWithAllParameters(self):
        gist = self.g.get_gist("2729810")
        gist.edit(
            "Description edited by PyGithub",
            {"barbaz.txt": github.InputFileContent("File also created by PyGithub")},
        )
        self.assertEqual(gist.description, "Description edited by PyGithub")
        self.assertEqual(
            gist.updated_at,
            datetime(2012, 5, 19, 7, 6, 10, tzinfo=timezone.utc),
        )
        self.assertEqual(set(gist.files.keys()), {"foobar.txt", "barbaz.txt"})

    def testDeleteFile(self):
        gist = self.g.get_gist("5339374")
        self.assertEqual(sorted(gist.files.keys()), ["bar.txt", "foo.txt"])
        gist.edit(files={"foo.txt": None})
        self.assertEqual(list(gist.files.keys()), ["bar.txt"])

    def testRenameFile(self):
        gist = self.g.get_gist("5339374")
        self.assertEqual(list(gist.files.keys()), ["bar.txt"])
        gist.edit(files={"bar.txt": github.InputFileContent(gist.files["bar.txt"].content, new_name="baz.txt")})
        self.assertEqual(list(gist.files.keys()), ["baz.txt"])

    def testCreateComment(self):
        gist = self.g.get_gist("2729810")
        comment = gist.create_comment("Comment created by PyGithub")
        self.assertEqual(comment.id, 323629)

    def testGetComments(self):
        gist = self.g.get_gist("2729810")
        self.assertListKeyEqual(gist.get_comments(), lambda c: c.id, [323637])

    def testStarring(self):
        gist = self.g.get_gist("2729810")
        self.assertFalse(gist.is_starred())
        gist.set_starred()
        self.assertTrue(gist.is_starred())
        gist.reset_starred()
        self.assertFalse(gist.is_starred())

    def testFork(self):
        gist = self.g.get_gist("6296553")  # Random gist
        myGist = gist.create_fork()
        self.assertEqual(myGist.id, "6296732")
        self.assertEqual(myGist.fork_of, None)  # WTF
        sameGist = self.g.get_gist("6296732")
        self.assertEqual(sameGist.fork_of.id, "6296553")

    def testDelete(self):
        gist = self.g.get_gist("2729810")
        gist.delete()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class Hook(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.hook = self.g.get_user().get_repo("PyGithub").get_hook(257993)

    def testAttributes(self):
        self.assertTrue(self.hook.active)  # WTF
        self.assertEqual(self.hook.config, {"url": "http://foobar.com"})
        self.assertEqual(
            self.hook.created_at,
            datetime(2012, 5, 19, 6, 1, 45, tzinfo=timezone.utc),
        )
        self.assertIsNone(self.hook.deliveries_url)
        self.assertEqual(self.hook.events, ["push"])
        self.assertEqual(self.hook.id, 257993)
        self.assertEqual(self.hook.last_response.status, "ok")
        self.assertEqual(self.hook.last_response.message, "OK")
        self.assertEqual(self.hook.last_response.code, 200)
        self.assertEqual(self.hook.name, "web")
        self.assertEqual(self.hook.ping_url, "https://api.github.com/repos/jacquev6/PyGithub/hooks/257993/pings")
        self.assertEqual(self.hook.test_url, "https://api.github.com/repos/jacquev6/PyGithub/hooks/257993/tests")
        self.assertIsNone(self.hook.type)
        self.assertEqual(
            self.hook.updated_at,
            datetime(2012, 5, 29, 18, 49, 47, tzinfo=timezone.utc),
        )
        self.assertEqual(self.hook.url, "https://api.github.com/repos/jacquev6/PyGithub/hooks/257993")
        self.assertEqual(
            self.hook.test_url,
            "https://api.github.com/repos/jacquev6/PyGithub/hooks/257993/tests",
        )
        self.assertEqual(
            self.hook.ping_url,
            "https://api.github.com/repos/jacquev6/PyGithub/hooks/257993/pings",
        )

        self.assertEqual(
            repr(self.hook),
            'Hook(url="https://api.github.com/repos/jacquev6/PyGithub/hooks/257993", id=257993)',
        )
        self.assertEqual(repr(self.hook.last_response), 'HookResponse(status="ok")')

    def testLazyAttributes(self):
        hook = self.g.withLazy(True).get_repo("lazy/repo").get_hook(42)
        self.assertEqual(str(hook), 'Hook(url="/repos/lazy/repo/hooks/42", id=42)')
        self.assertEqual(hook.id, 42)
        self.assertEqual(hook.url, "/repos/lazy/repo/hooks/42")

    def testEditWithMinimalParameters(self):
        self.hook.edit("web", {"url": "http://foobar.com/hook"})
        self.assertEqual(self.hook.config, {"url": "http://foobar.com/hook"})
        self.assertEqual(
            self.hook.updated_at,
            datetime(2012, 5, 19, 5, 8, 16, tzinfo=timezone.utc),
        )

    def testDelete(self):
        self.hook.delete()

    def testTest(self):
        self.hook.test()  # This does not update attributes of hook

    def testPing(self):
        self.hook.ping()  # This does not update attributes of hook

    def testEditWithAllParameters(self):
        self.hook.edit("web", {"url": "http://foobar.com"}, events=["fork", "push"])
        self.assertEqual(self.hook.events, ["fork", "push"])
        self.hook.edit("web", {"url": "http://foobar.com"}, add_events=["push"])
        self.assertEqual(self.hook.events, ["fork", "push"])
        self.hook.edit("web", {"url": "http://foobar.com"}, remove_events=["fork"])
        self.assertEqual(self.hook.events, ["push"])
        self.hook.edit("web", {"url": "http://foobar.com"}, active=True)
        self.assertTrue(self.hook.active)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Nicolas Agustín Torres <nicolastrres@gmail.com>               #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Huan-Cheng Chang <changhc84@gmail.com>                        #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Malik Shahzad Muzaffar <shahzad.malik.muzaffar@cern.ch>       #
# Copyright 2024 Arash Kadkhodaei <arash77.kad@gmail.com>                      #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class IssueComment(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.comment = self.g.get_repo("PyGithub/PyGithub").get_issue(28).get_comment(20227753)

    def testAttributes(self):
        self.assertEqual(self.comment.author_association, "CONTRIBUTOR")
        self.assertEqual(self.comment.body, "Comment created by PyGithub\n")
        self.assertIsNone(self.comment.body_html)
        self.assertIsNone(self.comment.body_text)
        self.assertEqual(self.comment.created_at, datetime(2013, 6, 29, 10, 31, 38, tzinfo=timezone.utc))
        self.assertEqual(self.comment.html_url, "https://github.com/PyGithub/PyGithub/issues/28#issuecomment-20227753")
        self.assertEqual(self.comment.id, 20227753)
        self.assertEqual(self.comment.issue_url, "https://api.github.com/repos/PyGithub/PyGithub/issues/28")
        self.assertEqual(self.comment.node_id, "MDEyOklzc3VlQ29tbWVudDIwMjI3NzUz")
        self.assertEqual(self.comment.body, "Comment created by PyGithub\n")
        self.assertEqual(self.comment.created_at, datetime(2013, 6, 29, 10, 31, 38, tzinfo=timezone.utc))
        self.assertEqual(self.comment.id, 20227753)
        self.assertIsNone(self.comment.performed_via_github_app)
        self.assertEqual(
            self.comment.reactions,
            {
                "url": "https://api.github.com/repos/PyGithub/PyGithub/issues/comments/20227753/reactions",
                "total_count": 2,
                "+1": 1,
                "-1": 0,
                "laugh": 0,
                "hooray": 1,
                "confused": 0,
                "heart": 0,
                "rocket": 0,
                "eyes": 0,
            },
        )
        self.assertEqual(self.comment.updated_at, datetime(2013, 6, 29, 10, 31, 38, tzinfo=timezone.utc))
        self.assertEqual(self.comment.url, "https://api.github.com/repos/PyGithub/PyGithub/issues/comments/20227753")
        self.assertEqual(self.comment.user.login, "stuglaser")
        self.assertEqual(self.comment.html_url, "https://github.com/PyGithub/PyGithub/issues/28#issuecomment-20227753")
        self.assertEqual(repr(self.comment), 'IssueComment(user=NamedUser(login="stuglaser"), id=20227753)')
        self.assertEqual(
            self.comment.reactions,
            {
                "url": "https://api.github.com/repos/PyGithub/PyGithub/issues/comments/20227753/reactions",
                "total_count": 2,
                "+1": 1,
                "-1": 0,
                "laugh": 0,
                "hooray": 1,
                "confused": 0,
                "heart": 0,
                "rocket": 0,
                "eyes": 0,
            },
        )

    def testLazyAttributes(self):
        comment = self.g.withLazy(True).get_repo("lazy/repo").get_issue(42).get_comment(24)
        self.assertEqual(str(comment), "IssueComment(user=None, id=24)")
        self.assertEqual(comment.id, 24)
        self.assertEqual(comment.url, "/repos/lazy/repo/issues/comments/24")

    def testEdit(self):
        self.comment.edit("Comment edited by PyGithub")
        self.assertEqual(self.comment.body, "Comment edited by PyGithub")
        self.assertEqual(
            self.comment.updated_at,
            datetime(2012, 5, 20, 11, 53, 59, tzinfo=timezone.utc),
        )

    def testMinimize(self):
        self.assertTrue(self.comment.minimize())

    def testUnminimize(self):
        self.assertTrue(self.comment.unminimize())

    def testGetReactions(self):
        reactions = self.comment.get_reactions()
        self.assertEqual(reactions[0].content, "+1")

    def testCreateReaction(self):
        reaction = self.comment.create_reaction("hooray")

        self.assertEqual(reaction.id, 17282654)
        self.assertEqual(reaction.content, "hooray")

    def testDeleteReaction(self):
        self.assertTrue(self.comment.delete_reaction(85743754))

    # this should be the last test as this deletes the comment used above.
    def testDelete(self):
        self.comment.delete()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Thomas Cooper <coopernetes@proton.me>                         #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import pytest

import github.DependabotAlert
import github.PaginatedList

from . import Framework


class DependabotAlert(Framework.TestCase):
    alert: github.DependabotAlert.DependabotAlert

    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("coopernetes/PyGithub")

    def testAttributes(self):
        alert = self.repo.get_dependabot_alert(1)
        self.assertIsNone(alert.auto_dismissed_at)
        self.assertEqual(alert.created_at, datetime(2024, 1, 20, 17, 12, 38, tzinfo=timezone.utc))
        self.assertEqual(alert.dependency.package.name, "jinja2")
        self.assertEqual(alert.dismissed_at, datetime(2024, 1, 21, 3, 35, 38, tzinfo=timezone.utc))
        self.assertEqual(alert.dismissed_by.login, "coopernetes")
        self.assertEqual(alert.dismissed_reason, "tolerable_risk")
        self.assertEqual(alert.dismissed_comment, "Example comment")
        self.assertIsNone(alert.fixed_at)
        self.assertEqual(alert.html_url, "https://github.com/coopernetes/PyGithub/security/dependabot/1")
        self.assertEqual(alert.number, 1)
        self.assertEqual(alert.security_advisory.ghsa_id, "GHSA-h5c8-rqwp-cp95")
        self.assertEqual(alert.security_vulnerability.package.name, "jinja2")
        self.assertEqual(alert.state, "dismissed")
        self.assertEqual(alert.dependency.package.ecosystem, "pip")
        self.assertEqual(alert.dependency.package.name, "jinja2")
        self.assertEqual(alert.dependency.manifest_path, "requirements/docs.txt")
        self.assertEqual(alert.dependency.scope, "runtime")
        self.assertEqual(alert.security_advisory.ghsa_id, "GHSA-h5c8-rqwp-cp95")
        self.assertEqual(alert.security_advisory.cve_id, "CVE-2024-22195")
        self.assertEqual(
            alert.security_advisory.summary,
            "Jinja vulnerable to HTML attribute injection when passing user input as keys to xmlattr filter",
        )
        self.assertEqual(
            alert.security_advisory.description,
            "The `xmlattr` filter in affected versions of Jinja accepts keys containing spaces. XML/HTML attributes cannot contain spaces, as each would then be interpreted as a separate attribute. If an application accepts keys (as opposed to only values) as user input, and renders these in pages that other users see as well, an attacker could use this to inject other attributes and perform XSS. Note that accepting keys as user input is not common or a particularly intended use case of the `xmlattr` filter, and an application doing so should already be verifying what keys are provided regardless of this fix.",
        )
        self.assertEqual(alert.security_advisory.severity, "medium")
        self.assertEqual(alert.security_advisory.identifiers[0]["value"], "GHSA-h5c8-rqwp-cp95")
        self.assertEqual(alert.security_advisory.identifiers[0]["type"], "GHSA")
        self.assertEqual(alert.security_advisory.identifiers[1]["value"], "CVE-2024-22195")
        self.assertEqual(alert.security_advisory.identifiers[1]["type"], "CVE")
        self.assertEqual(
            alert.security_advisory.references[0]["url"],
            "https://github.com/pallets/jinja/security/advisories/GHSA-h5c8-rqwp-cp95",
        )
        self.assertEqual(
            alert.security_advisory.references[1]["url"], "https://nvd.nist.gov/vuln/detail/CVE-2024-22195"
        )
        self.assertEqual(
            alert.security_advisory.references[2]["url"],
            "https://github.com/pallets/jinja/commit/716795349a41d4983a9a4771f7d883c96ea17be7",
        )
        self.assertEqual(
            alert.security_advisory.references[3]["url"], "https://github.com/pallets/jinja/releases/tag/3.1.3"
        )
        self.assertEqual(
            alert.security_advisory.references[4]["url"], "https://github.com/advisories/GHSA-h5c8-rqwp-cp95"
        )
        self.assertEqual(alert.security_advisory.published_at, datetime(2024, 1, 11, 15, 20, 48, tzinfo=timezone.utc))
        self.assertEqual(alert.security_advisory.updated_at, datetime(2024, 1, 11, 15, 20, 50, tzinfo=timezone.utc))
        self.assertEqual(alert.security_advisory.withdrawn_at, None)
        self.assertEqual(alert.security_advisory.vulnerabilities[0].package.ecosystem, "pip")
        self.assertEqual(alert.security_advisory.vulnerabilities[0].package.name, "jinja2")
        self.assertEqual(alert.security_advisory.vulnerabilities[0].vulnerable_version_range, "< 3.1.3")
        self.assertEqual(alert.security_advisory.vulnerabilities[0].severity, "medium")
        self.assertEqual(alert.security_advisory.vulnerabilities[0].first_patched_version["identifier"], "3.1.3")
        self.assertEqual(alert.security_vulnerability.package.ecosystem, "pip")
        self.assertEqual(alert.security_vulnerability.package.name, "jinja2")
        self.assertEqual(alert.security_vulnerability.vulnerable_version_range, "< 3.1.3")
        self.assertEqual(alert.security_vulnerability.severity, "medium")
        self.assertEqual(alert.security_vulnerability.first_patched_version["identifier"], "3.1.3")
        self.assertEqual(alert.updated_at, datetime(2024, 1, 21, 3, 35, 38, tzinfo=timezone.utc))
        self.assertEqual(alert.url, "https://api.github.com/repos/coopernetes/PyGithub/dependabot/alerts/1")
        self.assertEqual(alert.html_url, "https://github.com/coopernetes/PyGithub/security/dependabot/1")

    def testMultipleAlerts(self):
        multiple_alerts = self.repo.get_dependabot_alerts()
        self.assertIsInstance(multiple_alerts, github.PaginatedList.PaginatedList)
        self.assertIsInstance(multiple_alerts[0], github.DependabotAlert.DependabotAlert)
        alert_list = [alert for alert in multiple_alerts]
        test_alert = alert_list[0]

        self.assertEqual(len(alert_list), 1)
        # Everything below is the same as testAttributes. This is just to make sure the list works.
        self.assertEqual(test_alert.number, 1)
        self.assertEqual(test_alert.state, "dismissed")
        self.assertEqual(test_alert.dependency.package.ecosystem, "pip")
        self.assertEqual(test_alert.dependency.package.name, "jinja2")
        self.assertEqual(test_alert.dependency.manifest_path, "requirements/docs.txt")
        self.assertEqual(test_alert.dependency.scope, "runtime")
        self.assertEqual(test_alert.security_advisory.ghsa_id, "GHSA-h5c8-rqwp-cp95")
        self.assertEqual(test_alert.security_advisory.cve_id, "CVE-2024-22195")
        self.assertEqual(
            test_alert.security_advisory.summary,
            "Jinja vulnerable to HTML attribute injection when passing user input as keys to xmlattr filter",
        )
        self.assertEqual(
            test_alert.security_advisory.description,
            "The `xmlattr` filter in affected versions of Jinja accepts keys containing spaces. XML/HTML attributes cannot contain spaces, as each would then be interpreted as a separate attribute. If an application accepts keys (as opposed to only values) as user input, and renders these in pages that other users see as well, an attacker could use this to inject other attributes and perform XSS. Note that accepting keys as user input is not common or a particularly intended use case of the `xmlattr` filter, and an application doing so should already be verifying what keys are provided regardless of this fix.",
        )
        self.assertEqual(test_alert.security_advisory.severity, "medium")
        self.assertEqual(test_alert.security_advisory.identifiers[0]["value"], "GHSA-h5c8-rqwp-cp95")
        self.assertEqual(test_alert.security_advisory.identifiers[0]["type"], "GHSA")
        self.assertEqual(test_alert.security_advisory.identifiers[1]["value"], "CVE-2024-22195")
        self.assertEqual(test_alert.security_advisory.identifiers[1]["type"], "CVE")
        self.assertEqual(
            test_alert.security_advisory.published_at, datetime(2024, 1, 11, 15, 20, 48, tzinfo=timezone.utc)
        )
        self.assertEqual(
            test_alert.security_advisory.updated_at, datetime(2024, 1, 11, 15, 20, 50, tzinfo=timezone.utc)
        )
        self.assertEqual(test_alert.security_advisory.withdrawn_at, None)
        self.assertEqual(test_alert.security_advisory.vulnerabilities[0].package.ecosystem, "pip")
        self.assertEqual(test_alert.security_advisory.vulnerabilities[0].package.name, "jinja2")
        self.assertEqual(test_alert.security_advisory.vulnerabilities[0].vulnerable_version_range, "< 3.1.3")
        self.assertEqual(test_alert.security_advisory.vulnerabilities[0].severity, "medium")
        self.assertEqual(test_alert.security_advisory.vulnerabilities[0].first_patched_version["identifier"], "3.1.3")
        self.assertEqual(test_alert.security_vulnerability.package.ecosystem, "pip")
        self.assertEqual(test_alert.security_vulnerability.package.name, "jinja2")
        self.assertEqual(test_alert.security_vulnerability.vulnerable_version_range, "< 3.1.3")
        self.assertEqual(test_alert.security_vulnerability.severity, "medium")
        self.assertEqual(test_alert.security_vulnerability.first_patched_version["identifier"], "3.1.3")

        self.assertEqual(test_alert.url, "https://api.github.com/repos/coopernetes/PyGithub/dependabot/alerts/1")
        self.assertEqual(test_alert.html_url, "https://github.com/coopernetes/PyGithub/security/dependabot/1")
        self.assertEqual(test_alert.created_at, datetime(2024, 1, 20, 17, 12, 38, tzinfo=timezone.utc))
        self.assertEqual(test_alert.updated_at, datetime(2024, 1, 20, 22, 4, tzinfo=timezone.utc))
        self.assertEqual(test_alert.dismissed_at, datetime(2024, 1, 20, 22, 4, tzinfo=timezone.utc))
        self.assertEqual(test_alert.dismissed_by.login, "coopernetes")
        self.assertEqual(test_alert.dismissed_reason, "tolerable_risk")
        self.assertEqual(test_alert.dismissed_comment, "Example comment")
        self.assertEqual(test_alert.fixed_at, None)

    def testRepr(self):
        alert = self.repo.get_dependabot_alert(1)
        self.assertEqual(repr(alert), 'DependabotAlert(number=1, ghsa_id="GHSA-h5c8-rqwp-cp95")')

    def testGetAlertsWithAllArguments(self):
        alerts = self.repo.get_dependabot_alerts(
            "open", "medium", "pip", "foo,jinja2", "bar/package.json,requirements/docs.txt", "runtime", "created", "asc"
        )
        self.assertEqual(len(list(alerts)), 1)

    def testUpdateAlertDismissedWithoutReason(self):
        with pytest.raises(AssertionError):
            self.repo.update_dependabot_alert(1, "dismissed")

    def testUpdateAlertOpen(self):
        alert = self.repo.update_dependabot_alert(1, "open")
        self.assertEqual(alert.state, "open")
        self.assertEqual(alert.dismissed_reason, None)
        self.assertEqual(alert.dismissed_comment, None)

    def testUpdateAlertDismissed(self):
        alert = self.repo.update_dependabot_alert(1, "dismissed", "tolerable_risk", "Example comment")
        self.assertEqual(alert.state, "dismissed")
        self.assertEqual(alert.dismissed_reason, "tolerable_risk")
        self.assertEqual(alert.dismissed_comment, "Example comment")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Jonathan Leitschuh <jonathan.leitschuh@gmail.com>             #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github.RepositoryAdvisory

from . import Framework


class RepositoryAdvisory(Framework.TestCase):
    advisory: github.RepositoryAdvisory.RepositoryAdvisory

    def setUp(self):
        super().setUp()
        self.repo = self.g.get_user().get_repo("security-research")
        self.advisory = self.repo.get_repository_advisory("GHSA-wmmh-r9w4-hpxx")
        self.advisory.clear_credits()
        self.advisory.offer_credit("octocat", "analyst")

    def testAttributes(self):
        self.assertEqual(self.advisory.author.login, "JLLeitschuh")
        self.assertEqual(self.advisory.closed_at, None)
        self.assertIsNone(self.advisory.collaborating_teams)
        self.assertIsNone(self.advisory.collaborating_users)
        self.assertEqual(
            self.advisory.created_at,
            datetime(2023, 3, 28, 21, 41, 40, tzinfo=timezone.utc),
        )
        self.assertListKeyEqual(self.advisory.credits, lambda e: (e.login, e.type), [("octocat", "analyst")])
        self.assertListKeyEqual(
            self.advisory.credits_detailed,
            lambda e: (e.user.login, e.type),
            [("octocat", "analyst")],
        )
        self.assertEqual(self.advisory.cve_id, "CVE-2023-00000")
        self.assertEqual(self.advisory.cvss.vector_string, "CVSS:3.1/AV:N/AC:H/PR:H/UI:R/S:C/C:H/I:H/A:H")
        self.assertIsNone(self.advisory.cvss_severities)
        self.assertListEqual(self.advisory.cwe_ids, ["CWE-400", "CWE-501"])
        self.assertListKeyEqual(
            self.advisory.cwes,
            lambda e: (e.cwe_id, e.name),
            [
                ("CWE-400", "Uncontrolled Resource Consumption"),
                ("CWE-501", "Trust Boundary Violation"),
            ],
        )
        self.assertEqual(
            self.advisory.description,
            "This is a detailed description of this advisories impact and patches.",
        )
        self.assertEqual(self.advisory.ghsa_id, "GHSA-wmmh-r9w4-hpxx")
        self.assertEqual(
            self.advisory.html_url,
            "https://github.com/JLLeitschuh/security-research/security/advisories/GHSA-wmmh-r9w4-hpxx",
        )
        self.assertEqual(
            self.advisory.identifiers,
            [{"value": "GHSA-wmmh-r9w4-hpxx", "type": "GHSA"}, {"value": "CVE-2023-00000", "type": "CVE"}],
        )
        self.assertIsNone(self.advisory.private_fork)
        self.assertEqual(self.advisory.published_at, None)
        self.assertIsNone(self.advisory.publisher)
        self.assertEqual(self.advisory.severity, "high")
        self.assertEqual(self.advisory.state, "draft")
        self.assertIsNone(self.advisory.submission)
        self.assertEqual(self.advisory.summary, "A test creating a GHSA via the API")
        self.assertEqual(
            self.advisory.updated_at,
            datetime(2023, 3, 30, 19, 31, 33, tzinfo=timezone.utc),
        )
        self.assertEqual(
            self.advisory.url,
            "https://api.github.com/repos/JLLeitschuh/security-research/security-advisories/GHSA-wmmh-r9w4-hpxx",
        )
        self.assertListKeyEqual(
            self.advisory.vulnerabilities,
            lambda e: (
                (e.package.ecosystem, e.package.name),
                e.patched_versions,
                e.vulnerable_functions,
                e.vulnerable_version_range,
            ),
            [(("npm", "a-package"), "1.0.5", ["function-name"], ">= 1.0.2")],
        )
        self.assertEqual(self.advisory.withdrawn_at, None)

    def testRemoveCredit(self):
        self.advisory.revoke_credit("octocat")
        self.assertListKeyEqual(
            self.advisory.credits,
            lambda e: e.login,
            [],
        )
        self.assertListKeyEqual(
            self.advisory.credits_detailed,
            lambda e: e.user.login,
            [],
        )

    def testOfferCredit(self):
        self.advisory.offer_credit("JLLeitschuh", "reporter")
        self.assertListKeyEqual(
            self.advisory.credits,
            lambda e: e.login,
            ["octocat", "JLLeitschuh"],
        )
        self.assertListKeyEqual(
            self.advisory.credits_detailed,
            lambda e: e.user.login,
            ["octocat", "JLLeitschuh"],
        )

    def testOfferCredits(self):
        self.advisory.clear_credits()
        self.advisory.offer_credits(
            [
                {"login": "octocat", "type": "sponsor"},
                {"login": "JLLeitschuh", "type": "reporter"},
            ]
        )
        self.assertListKeyEqual(
            self.advisory.credits_detailed,
            lambda e: (e.user.login, e.type),
            [("octocat", "sponsor"), ("JLLeitschuh", "reporter")],
        )

    def testRepositoryWithNoAdvisories(self):
        repo = self.g.get_user().get_repo("PyGithub")
        self.assertListKeyEqual(
            repo.get_repository_advisories(),
            lambda e: e.ghsa_id,
            [],
        )

    def testGetAdvisories(self):
        self.assertListKeyEqual(
            self.repo.get_repository_advisories(),
            lambda e: e.ghsa_id,
            [
                "GHSA-wmmh-r9w4-hpxx",
                "GHSA-wvgm-59wj-rh8h",
                "GHSA-22cq-8f5q-p5g2",
                "GHSA-7hfp-mpq6-2jhf",
                "GHSA-hfmw-fx2m-jj4c",
                "GHSA-rvp4-r3g6-8hxq",
                "GHSA-cm59-pr5q-cw85",
                "GHSA-vpcc-9rh2-8jfp",
                "GHSA-7fjx-657r-9r5h",
                "GHSA-22c6-wcjm-qfjg",
                "GHSA-5w9v-8x7x-rfqm",
                "GHSA-2r85-x9cf-8fcg",
                "GHSA-6m9h-r5m3-9r7f",
                "GHSA-f4jh-ww96-9h9j",
                "GHSA-j83w-7qr9-wv86",
                "GHSA-7gf3-89f6-823j",
                "GHSA-jpcm-4485-69p7",
            ],
        )

    def testCreateRepositoryAdvisory(self):
        repo = self.g.get_repo("JLLeitschuh/code-sandbox")
        advisory = repo.create_repository_advisory(
            "A test creating a GHSA via the API",
            "This is a detailed description of this advisories impact and patches.",
            "high",
            "CVE-2000-00000",
            vulnerabilities=[
                {
                    "package": {"ecosystem": "npm", "name": "b-package"},
                    "vulnerable_version_range": "<=4.0.4",
                    "patched_versions": "4.0.5",
                    "vulnerable_functions": ["function-name"],
                }
            ],
            cwe_ids=["CWE-401", "CWE-502"],
            credits=[
                {"login": "octocat", "type": "analyst"},
                {"login": "JLLeitschuh", "type": "reporter"},
            ],
        )
        self.assertEqual(advisory.ghsa_id, "GHSA-g45c-2crh-4xmp")
        self.assertEqual(advisory.summary, "A test creating a GHSA via the API")
        self.assertEqual(
            advisory.description,
            "This is a detailed description of this advisories impact and patches.",
        )
        self.assertEqual(advisory.severity, "high")
        self.assertEqual(advisory.cve_id, "CVE-2000-00000")
        self.assertListKeyEqual(
            advisory.vulnerabilities,
            lambda e: (
                (e.package.ecosystem, e.package.name),
                e.patched_versions,
                e.vulnerable_functions,
                e.vulnerable_version_range,
            ),
            [(("npm", "b-package"), "4.0.5", ["function-name"], "<=4.0.4")],
        )
        self.assertListKeyEqual(
            advisory.cwe_ids,
            lambda e: e,
            ["CWE-401", "CWE-502"],
        )
        self.assertListKeyEqual(
            advisory.credits_detailed,
            lambda e: (e.user.login, e.type),
            [("octocat", "analyst"), ("JLLeitschuh", "reporter")],
        )

    def testUpdateRepositoryAdvisory(self):
        repo = self.g.get_repo("JLLeitschuh/code-sandbox")
        advisory = repo.get_repository_advisory("GHSA-g45c-2crh-4xmp")
        advisory.edit(
            summary="A test updating a GHSA via the API",
            description="This is an updated detailed description of this advisories impact and patches.",
            severity_or_cvss_vector_string="low",
            cve_id="CVE-2000-00001",
            vulnerabilities=[
                {
                    "package": {"ecosystem": "npm", "name": "c-package"},
                    "vulnerable_version_range": "<=4.0.6",
                    "patched_versions": "4.0.7",
                    "vulnerable_functions": ["function-name-a"],
                }
            ],
            cwe_ids=["CWE-402", "CWE-500"],
            credits=[
                {"login": "octocat", "type": "sponsor"},
                {"login": "JLLeitschuh", "type": "reporter"},
            ],
        )
        self.assertEqual(advisory.ghsa_id, "GHSA-g45c-2crh-4xmp")
        self.assertEqual(advisory.summary, "A test updating a GHSA via the API")
        self.assertEqual(
            advisory.description,
            "This is an updated detailed description of this advisories impact and patches.",
        )
        self.assertEqual(advisory.severity, "low")
        self.assertEqual(advisory.cve_id, "CVE-2000-00001")
        self.assertListKeyEqual(
            advisory.vulnerabilities,
            lambda e: (
                (e.package.ecosystem, e.package.name),
                e.patched_versions,
                e.vulnerable_functions,
                e.vulnerable_version_range,
            ),
            [(("npm", "c-package"), "4.0.7", ["function-name-a"], "<=4.0.6")],
        )
        self.assertListKeyEqual(
            advisory.cwe_ids,
            lambda e: e,
            ["CWE-402", "CWE-500"],
        )
        self.assertListKeyEqual(
            advisory.credits_detailed,
            lambda e: (e.user.login, e.type),
            [("octocat", "sponsor"), ("JLLeitschuh", "reporter")],
        )

    def testUpdateSingleFieldDoesNotRemoveOtherFields(self):
        repo = self.g.get_repo("JLLeitschuh/code-sandbox")
        advisory = repo.create_repository_advisory(
            "A test editing a GHSA via the API with only a single manipulation",
            "This is a detailed description of this advisories impact and patches.",
            "high",
            "CVE-2000-00000",
            vulnerabilities=[
                {
                    "package": {"ecosystem": "npm", "name": "b-package"},
                    "vulnerable_version_range": "<=4.0.4",
                    "patched_versions": "4.0.5",
                    "vulnerable_functions": ["function-name"],
                }
            ],
            cwe_ids=["CWE-401", "CWE-502"],
            credits=[
                {"login": "octocat", "type": "analyst"},
                {"login": "JLLeitschuh", "type": "reporter"},
            ],
        )
        advisory.edit(description="A modified description")
        self.assertEqual(advisory.ghsa_id, "GHSA-4wwp-8jp9-9233")
        self.assertEqual(
            advisory.summary,
            "A test editing a GHSA via the API with only a single manipulation",
        )
        self.assertEqual(advisory.description, "A modified description")
        self.assertEqual(advisory.severity, "high")
        self.assertEqual(advisory.cve_id, "CVE-2000-00000")
        self.assertListKeyEqual(
            advisory.vulnerabilities,
            lambda e: (
                (e.package.ecosystem, e.package.name),
                e.patched_versions,
                e.vulnerable_functions,
                e.vulnerable_version_range,
            ),
            [(("npm", "b-package"), "4.0.5", ["function-name"], "<=4.0.4")],
        )
        self.assertListKeyEqual(
            advisory.cwe_ids,
            lambda e: e,
            ["CWE-401", "CWE-502"],
        )
        self.assertListKeyEqual(
            advisory.credits_detailed,
            lambda e: (e.user.login, e.type),
            [("octocat", "analyst"), ("JLLeitschuh", "reporter")],
        )

    def testAddVulnerability(self):
        repo = self.g.get_repo("JLLeitschuh/code-sandbox")
        advisory = repo.create_repository_advisory(
            summary="A test creating a GHSA via the API adding and removing vulnerabilities",
            description="Simple description",
            severity_or_cvss_vector_string="low",
        )
        advisory.add_vulnerability(ecosystem="maven")
        self.assertListKeyEqual(
            advisory.vulnerabilities,
            lambda e: (
                (e.package.ecosystem, e.package.name),
                e.patched_versions,
                e.vulnerable_functions,
                e.vulnerable_version_range,
            ),
            [(("maven", None), None, [], None)],
        )
        advisory.add_vulnerability(
            ecosystem="npm",
            package_name="b-package",
            vulnerable_version_range="<=4.0.9",
            patched_versions="4.0.10",
            vulnerable_functions=["function-name-c"],
        )
        self.assertListKeyEqual(
            advisory.vulnerabilities,
            lambda e: (
                (e.package.ecosystem, e.package.name),
                e.patched_versions,
                e.vulnerable_functions,
                e.vulnerable_version_range,
            ),
            [
                (("maven", None), None, [], None),
                (("npm", "b-package"), "4.0.10", ["function-name-c"], "<=4.0.9"),
            ],
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from datetime import datetime, timezone

from . import Framework


class Download(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.download = self.g.get_user().get_repo("PyGithub").get_download(242550)

    def testAttributes(self):
        self.assertEqual(self.download.accesskeyid, None)
        self.assertEqual(self.download.acl, None)
        self.assertEqual(self.download.bucket, None)
        self.assertEqual(self.download.content_type, "text/plain")
        self.assertEqual(
            self.download.created_at,
            datetime(2012, 5, 22, 18, 58, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(self.download.description, None)
        self.assertEqual(self.download.download_count, 0)
        self.assertEqual(self.download.expirationdate, None)
        self.assertEqual(
            self.download.html_url,
            "https://github.com/downloads/jacquev6/PyGithub/Foobar.txt",
        )
        self.assertEqual(self.download.id, 242550)
        self.assertEqual(self.download.mime_type, None)
        self.assertEqual(self.download.name, "Foobar.txt")
        self.assertEqual(self.download.path, None)
        self.assertEqual(self.download.policy, None)
        self.assertEqual(self.download.prefix, None)
        self.assertEqual(self.download.redirect, None)
        self.assertEqual(self.download.s3_url, None)
        self.assertEqual(self.download.signature, None)
        self.assertEqual(self.download.size, 1024)
        self.assertEqual(
            self.download.url,
            "https://api.github.com/repos/jacquev6/PyGithub/downloads/242550",
        )
        self.assertEqual(repr(self.download), "Download(id=242550)")

    def testLazyAttributes(self):
        download = self.g.withLazy(True).get_repo("lazy/repo").get_download(42)
        self.assertEqual(str(download), "Download(id=42)")
        self.assertEqual(download.id, 42)
        self.assertEqual(download.url, "/repos/lazy/repo/downloads/42")

    def testDelete(self):
        self.download.delete()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from datetime import datetime, timezone

from . import Framework


class Authorization(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.authorization = self.g.get_user().get_authorization(372259)

    def testAttributes(self):
        self.assertEqual(
            self.authorization.app.url,
            "http://developer.github.com/v3/oauth/#oauth-authorizations-api",
        )
        self.assertEqual(self.authorization.app.name, "GitHub API")
        self.assertEqual(
            self.authorization.created_at,
            datetime(2012, 5, 22, 18, 3, 17, tzinfo=timezone.utc),
        )
        self.assertEqual(self.authorization.id, 372259)
        self.assertEqual(self.authorization.note, None)
        self.assertEqual(self.authorization.note_url, None)
        self.assertEqual(self.authorization.scopes, [])
        self.assertEqual(self.authorization.token, "82459c4500086f8f0cc67d2936c17d1e27ad1c33")
        self.assertEqual(
            self.authorization.updated_at,
            datetime(2012, 5, 22, 18, 3, 17, tzinfo=timezone.utc),
        )
        self.assertEqual(self.authorization.url, "https://api.github.com/authorizations/372259")
        self.assertEqual(repr(self.authorization), "Authorization(scopes=[])")
        self.assertEqual(repr(self.authorization.app), 'AuthorizationApplication(name="GitHub API")')

    def testEdit(self):
        self.authorization.edit()
        self.assertEqual(self.authorization.scopes, [])
        self.authorization.edit(scopes=["user"])
        self.assertEqual(self.authorization.scopes, ["user"])
        self.authorization.edit(add_scopes=["repo"])
        self.assertEqual(self.authorization.scopes, ["user", "repo"])
        self.authorization.edit(remove_scopes=["repo"])
        self.assertEqual(self.authorization.scopes, ["user"])
        self.assertEqual(self.authorization.note, None)
        self.assertEqual(self.authorization.note_url, None)
        self.authorization.edit(
            note="Note created by PyGithub",
            note_url="http://vincent-jacques.net/PyGithub",
        )
        self.assertEqual(self.authorization.note, "Note created by PyGithub")
        self.assertEqual(self.authorization.note_url, "http://vincent-jacques.net/PyGithub")

    def testDelete(self):
        self.authorization.delete()
